@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2023 Zoltan Turanyi
+	Copyright (C) 2008-2024 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -79,8 +79,17 @@ struct ElementGUIState {
     bool collapsed;                //If true, the GUI wants to collapse this element, else expand.
     bool element_exists = false;   //Set to true during compilation if one element recognizes itself
 };
+
+struct InlinedGUIState {
+    std::string gui_state;
+    bool element_exists = false;
+};
+
 /** A GUI Collapse/Expand catalog. Maps a unique name to a 'collapsed' flag. */
-using GUIState = std::unordered_map<std::string, ElementGUIState>;
+struct GUIState {
+    std::unordered_map<std::string, ElementGUIState> collapse;
+    std::unordered_map<std::string, InlinedGUIState> inlined;
+};
 
 struct ArrowAttrHelper {
     gsl::owner<AttributeList*> al;
@@ -108,6 +117,28 @@ struct LayoutStat {
 };
 
 
+    /** A group of arrow ends(false = start, true = end) that overlap */
+struct ArrowEndGroup
+{
+    struct ArrowEnd
+    {
+        Arrow* pA;
+        bool is_end;
+        bool is_shifting; //If true, this end needs to be shifted at de-overlapping
+        bool is_originally_shifting; //This is used to preserve the "is_shifting" across permutations
+        bool operator<(const ArrowEnd &o) const noexcept { return std::tie(pA, is_end)<std::tie(o.pA, o.is_end); }
+        const Arrow::Ending& ending() const noexcept { return pA->ends[is_end]; }
+    };
+    std::vector<ArrowEnd> ends; 
+    XY shift_vector;    ///<A 1-length vector perpendicular to the arrow endings. This is the direction to shift arrow ends.
+    void Add(Arrow* pA, bool is_end);
+    void ApplyShift();
+    size_t number_of_permutations() const;
+    void next_permutation(unsigned permutation_no);
+};
+
+using ArrowToScore = std::unordered_map<Arrow*, PathScore>;
+
 /** The Block Diagram chart class.*/
 class BlockChart : public ChartBase<BlockContext, BlockProgress>
 {
@@ -120,7 +151,7 @@ public:
     std::string GetLanguageDefaultText() const override { return "#This is the default block text\n"; } ///<Returns the default text of the chart in UTF-8
     bool GetLanguageHasAutoheading() const override { return false; } ///<True if the chart has automatic headings
     bool GetLanguageHasElementControls() const override { return false; } ///<True if the elements of the chart support GUI controls
-    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc) const override;
+    std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc, const LanguageCollection* languages) const override;
     std::map<std::string, std::string> RegisterLibraries() const override;
     /** @} */
     BlockStyle parent_style;         ///<Attributes the parent applies to its content (adds to BlockContext::running_style)
@@ -130,6 +161,7 @@ public:
 protected:
     unsigned                 block_seq;   ///<The next seq num for unique names
     std::map<std::string, std::unique_ptr<BlockBlock>, std::less<>> Templates;
+    bool has_xy_correlation = false;  ///<True if the chart has correlation between X and Y dimensions, e.g., the width of an element depends on the height of another.
 public:
     /** How many bulk layout block items do we register per block. This is a granularity thing as
      * we don't actually progress layout by blocks, but by constraints.*/
@@ -157,13 +189,13 @@ public:
 	static constexpr double def_multi_offset = 5;       ///<The default offset for multi-blocks
     EAlignPrio conflict_report;    ///<Warn of alignment conflicts of this level or more serious. We always report Explicit_low_low and above as errors.
 
-    BlockChart(FileReadProcedure *p, void *param);
+    BlockChart(FileReadProcedure *p, void *param, const LanguageCollection* languages);
     ~BlockChart();
 
     std::unique_ptr<BlockInstrList> PopContext();            ///<Closes a scope
 
-    static std::unique_ptr<Chart> Factory(FileReadProcedure *p, void *param)
-        { return std::make_unique<BlockChart>(p, param); }
+    static std::unique_ptr<Chart> Factory(FileReadProcedure *p, void *param, const LanguageCollection* languages)
+        { return std::make_unique<BlockChart>(p, param, languages); }
 
     bool AddCommandLineArg(const std::string & arg) override;
     void AddCommandLineOption(const Attribute & a) override;
@@ -172,7 +204,7 @@ public:
     bool ControlClicked(Element *, EGUIControlType) override;
     bool ApplyForcedDesign(const string & name) override;
     bool GetPedantic() const noexcept override { return Contexts.empty() ? true : Contexts.back().pedantic.value_or(true); }
-    void SetPedantic(bool pedantic) noexcept override { if (Contexts.size()) Contexts.back().pedantic = pedantic; }
+    void SetPedantic(bool pedantic) noexcept override { if (Contexts.size()) Contexts.back().pedantic = pedantic; Chart::SetPedantic(pedantic); }
     std::string_view GetCurrentPrefix() const noexcept { return Contexts.size() ? Contexts.back().GetPrefix() : std::string_view{}; }
 
     bool AddAttribute(const Attribute&);
@@ -184,11 +216,10 @@ public:
     /** Adds the list valid values for a chart options to a Csh object. */
     static bool AttributeValues(std::string_view attr, Csh &csh);
     /** Adds an attribute list to each of a list of styles.*/
-    void AddAttributeListToStyleListWithFilePos(gsl::owner<AttributeList *> al,
-                                                const StringWithPosList *styles);
-    void AddAttributeListToRunningStyle(gsl::owner<AttributeList *> al,
-                                        EUseKeywords e);
-    unsigned ParseText(std::string_view input, std::string_view filename) override;
+    void AddAttributeListToStyleListWithFilePos(gsl::owner<AttributeList *> al, const StringWithPosList *styles);
+    void AddAttributeListToRunningStyle(gsl::owner<AttributeList *> al, EUseKeywords e);
+    void SetXYCorrelation() { has_xy_correlation = true; }
+    void ParseText(std::string_view input, const FileLineCol& first_char) override;
     gsl::owner<BlockInstrList *> AppendInstructionToList(gsl::owner<BlockInstrList *>, gsl::owner<BlockInstruction *> b);
     void MoveArrowLabels(BlockInstrList &al);
     std::string CreateUniqueName(std::string_view name_original, const FileLineCol &l)
@@ -225,12 +256,18 @@ public:
 	void IncreaseTotal(const Block &b) { total += b; }
 	void IncreaseTotal(const XY &p) { total += p; }
 	OptAttr<double> GetCoord(const AlignTo &a, bool y, const BlockBlock *parent) const;
-    std::vector<Arrow*> CheckArrowOverlap();
+    std::vector<ArrowEndGroup> CheckArrowOverlap();
+    std::unordered_map<Arrow*, std::vector<Arrow*>> MergeFullOverlapping(std::vector<ArrowEndGroup>& groups);
+    ArrowToScore Redo(Canvas& canvas, const std::vector<Arrow*>& redo);
+    std::pair<ArrowToScore, std::vector<Arrow*>> RedoLayout(Canvas& canvas, bool bail_out_early);
+    void HandleArrowOverlap(Canvas& canvas, std::vector<ArrowEndGroup>&& groups);
+
     void CompleteParse(bool autoPaginate = false,
                        bool addHeading = true, XY pageSize = XY(0, 0),
                        bool fitWidth = true, bool collectLinkInfo = false) override;
     void CollectIsMapElements(Canvas & canvas) override;
     void RegisterAllLabels() override;
+    void Draw(Canvas& canvas, bool pageBreaks, unsigned page) override;
     void DrawComplete(Canvas & canvas, bool pageBreaks, unsigned page) override;
     void SetToEmpty() override;
 };
@@ -240,12 +277,12 @@ public:
  * @param [in] spec_blocks The blocks for first, prev, next, last. Either or all may be null meaning
  *                    No such block. The last block ([4]) is the block we do the resolving for.
  *                    It may also be null if we do resolving for e.g., an arrow.
- * @param [in] break_msg If non empty, we emit an error if a blockname refer to a "break block". Else we
+ * @param [in] break_msg If non empty, we emit an error if a block name refer to a "break block". Else we
  *                       remove that silently. You can use %b to replace to the block name .
- * @param [in] ambigous_msg If a block name is a special name, but the corresponding element in 'spec_blocks'
+ * @param [in] ambiguous_msg If a block name is a special name, but the corresponding element in 'spec_blocks'
  *                          is nullptr, we emit this message and clear the block. Else we silently clear the block.
  *                          You can use %b to replace to the block name and %p to replace to first or last.
- * @param [in] spec_block_msg If non empty, we emit an error if a blockname is a special block instead of
+ * @param [in] spec_block_msg If non empty, we emit an error if a block name is a special block instead of
  *                            resolving it. In this case the spec_block[0..3] is ignored. The name is removed
  *                            from the list.
  *                            You can use %b to replace to the block name.
@@ -256,7 +293,7 @@ public:
  * @returns if we have emitted any error.*/
 template <typename str_pos_vector>
 inline bool BlockElement::Resolve(str_pos_vector &blocks, SpecBlockPtrs spec_blocks,
-                                  std::string_view break_msg, std::string_view ambigous_msg,
+                                  std::string_view break_msg, std::string_view ambiguous_msg,
                                   std::string_view spec_block_msg, std::string *text, bool *justify)
 {
     //remove duplicates from the list
@@ -291,8 +328,8 @@ inline bool BlockElement::Resolve(str_pos_vector &blocks, SpecBlockPtrs spec_blo
                     }
                 } else if (spec_blocks[u]==nullptr || spec_blocks[u] == this) { //if 'prev' or 'first' is applied to the first block or 'next' and 'last' to the last one...
                     if (sp.file_pos.start.IsValid()) {//...and we know the location...
-                        if (ambigous_msg.length()) {  //...and this attribute is explicitly set at the block, not from a style (so we have ambigous_msg) ...
-                            std::string msg(ambigous_msg);
+                        if (ambiguous_msg.length()) {  //...and this attribute is explicitly set at the block, not from a style (so we have ambiguous_msg) ...
+                            std::string msg(ambiguous_msg);
                             //...then emit an error.
                             auto at = msg.find("%b");
                             if (at!=std::string::npos)
@@ -303,7 +340,7 @@ inline bool BlockElement::Resolve(str_pos_vector &blocks, SpecBlockPtrs spec_blo
                             chart.Error.Error(sp.file_pos.start, msg);
                             ret = true;
                         }
-                        //else (if this comes from a style or we for some reason dont know the location, ignore silently.
+                        //else (if this comes from a style or we for some reason don't know the location, ignore silently.
                     }
                     sp.name.clear();
                 } else {
@@ -329,7 +366,7 @@ inline bool BlockElement::Resolve(str_pos_vector &blocks, SpecBlockPtrs spec_blo
  * @param blocks The list of blocks to check and prune.
  * @param whattoignore What shall we say, what will be ignored if the block is not found?
  * @param [in] bool resolve_hidden What to do with hidden blocks.
- * @param [in] self_msg If non empty, we emit an error if a blockname refers to us.
+ * @param [in] self_msg If non empty, we emit an error if a block name refers to us.
  *                      Else we silently ignore and remove from list. Use %b to refer to the block name.
  *                      and %B to refer to the block that is collapsed and hides the one in question.
  * @param [in] keep_last If true, we keep the last one of blocks mentioned several times, else the first.
