@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -30,8 +30,8 @@
 
 using namespace block;
 
-BlockCsh::BlockCsh(Csh::FileListProc proc) :
-    Csh(BlockContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc)
+BlockCsh::BlockCsh(Csh::FileListProc proc, const LanguageCollection* languages) :
+    Csh(BlockContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc, languages)
 {
     ForbiddenStyles.erase("row");
     ForbiddenStyles.erase("col");
@@ -246,6 +246,9 @@ void BlockCsh::AddKeywordsToHints(bool include_dir, bool include_join_around, bo
     AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "shape",
         "Define a new block with a specific shape.",
         EHintType::KEYWORD, true, CshHintGraphicCallbackForKeywords));
+    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "space",
+                       "Add a bit of an empty space. (10 points by default, add a number.)",
+                       EHintType::KEYWORD, true, CshHintGraphicCallbackForKeywords));
     if (include_dir) {
         AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "leftof",
                            "Makes the block to be laid out left of (an)other block(s).",
@@ -434,9 +437,9 @@ void BlockCsh::AddOptionsToHints()
 }
 
 
-void BlockCsh::BeforeYaccParse(std::string&& input, int cursor_p)
+void BlockCsh::BeforeYaccParse(std::string&& input, CharByteIndex first_p, CharByteIndex cursor_p)
 {
-    Csh::BeforeYaccParse(std::move(input), cursor_p);
+    Csh::BeforeYaccParse(std::move(input), first_p, cursor_p);
     parent_name.clear();
     prefix_at_hint_loc.clear();
     Context2.clear();
@@ -453,15 +456,17 @@ void BlockCsh::BeforeYaccParse(std::string&& input, int cursor_p)
 /** Parse chart text for color syntax and hint info
  *
  * @param [in] input The chart text
+ * @param [in] first_char Position of the first character of 'input' in the file we parse.
+ *             If we parse a whole file, it shall be {0,0}.
  * @param [in] cursor_p The current position of the cursor.
  * @param [in] pedantic The initial value of the pedantic chart option.*/
-void BlockCsh::ParseText(std::string&& input, int cursor_p, bool pedantic)
+void BlockCsh::ParseText(std::string&& input, CharByteIndex first_char, CharByteIndex cursor_p, bool pedantic)
 {
     //initialize data struct
-    BeforeYaccParse(std::move(input), cursor_p);
+    BeforeYaccParse(std::move(input), first_char, cursor_p);
     Context2.back().pedantic = pedantic;
     //call parsing
-    sv_reader<true> reader(input_text, 0);
+    sv_reader<true> reader(input_text, first_char);
     block_csh_parse(*this, reader); //return value ignored
     //Tidy up afterwards
     AfterYaccParse();
@@ -586,8 +591,8 @@ bool BlockCsh::AddCSH_NewBlock(const CshPos &pos, std::string_view pname, int sh
 {
     if (pname.empty()) {
         parent_name = GetCurrentPrefix();
-        if (pos.first_pos<=pos.last_pos)
-            AddCSH(pos, COLOR_ENTITYNAME); //probably ""
+        if (!pos.empty())
+            AddCSH(pos, COLOR_ENTITYNAME); //probably pname = "" (two double quotes)
         return true;
     }
     if (shape<0) shape = -1; //box shape
@@ -1013,7 +1018,7 @@ void BlockCsh::DropBlockMentions(CshPos pos) {
  * @param [in] l The list contains tokens (names) and their positions.
  *               Names may be empty, in this case an error occurred there and
  *               we don't need to color.
- * @param [in] pos The whole span we color. We shall color before and after
+ * @param [in] pos The whole span we color. We shall also color before and after
  *                 any blocks within this position range.
  * @param [in] cname Color names to this color (except empty ones)
  *                   If COLOR_ENTITYNAME, we color the strings as block names
@@ -1029,11 +1034,11 @@ void BlockCsh::AddCSH_NameList(const CshStringViewWithPosList* l, const CshPos&
                                EColorSyntaxType cnumber, EColorSyntaxType cinstead)
 {
     if (l==nullptr) return;
-    int after_prev_last = pos.first_pos;
+    CharByteIndex after_prev_last = pos.first_pos;
     double dummy;
     for (auto &s : *l) {
         if (after_prev_last<s.file_pos.first_pos)
-            AddCSH(CshPos(after_prev_last, s.file_pos.first_pos-1), cbetween);
+            AddCSH(CshPos(after_prev_last, s.file_pos.first_pos), cbetween);
         if (cnumber!=COLOR_ERROR && !from_chars(s.name, dummy))
             AddCSH(s.file_pos, cnumber);
         else if (cname!=COLOR_ENTITYNAME)
@@ -1043,92 +1048,89 @@ void BlockCsh::AddCSH_NameList(const CshStringViewWithPosList* l, const CshPos&
                              l->size()==1 ? cinstead : COLOR_ERROR);
         else
             AddCSH_BlockNameAsAttrValue(s.file_pos, s.name);
-        after_prev_last = s.file_pos.last_pos+1;
+        after_prev_last = s.file_pos.end_pos;
     }
-    if (after_prev_last<=pos.last_pos)
-        AddCSH(CshPos(after_prev_last, pos.last_pos), cbetween);
+    if (after_prev_last<pos.end_pos)
+        AddCSH(CshPos(after_prev_last, pos.end_pos), cbetween);
 }
 
-std::pair<std::string_view, size_t> BlockCsh::EntityNameUnder(long pos) const {
+Csh::EntityNameUnderResult BlockCsh::EntityNameUnder(long pos) const {
+    if (auto res = EntityNameUnderInLined(pos)) return res;
     auto i = std::find_if(BlockMentions.begin(), BlockMentions.end(),
-                          [pos](const BlockMention &p) { return p.pos.IsWithin(pos); });
+                          [pos](const BlockMention &p) { return p.pos.IsWithinChar(pos); });
     if (i==BlockMentions.end())
-        return {{}, 0};
-    auto [entity_in_src, pos_in_src_in_entity] = Csh::EntityNameUnder(pos);
+        return {};
+    EntityNameUnderResult res = Csh::EntityNameUnder(pos);
     //the full block name must end with what is in the text file.
-    const int prefix_len = int(i->name.length())-int(entity_in_src.length());
-    if (prefix_len<0 || std::string_view(i->name).substr(prefix_len) != entity_in_src) {
+    const int prefix_len = int(i->name.length())-int(res.full_entity_name.length());
+    if (prefix_len<0 || std::string_view(i->name).substr(prefix_len) != res.full_entity_name) {
         _ASSERT(0);
-        return {{}, 0};
+        return {};
     }
     //1. Trim away parts that are not clicked on by the user. E.g., For A.B.C if the user
     // clicks on 'B', we just need to return A.B (even if in the text we only have 'B.C')
-    const auto bytes = GetUTF8ByteIndex(i->name, pos-i->pos.first_pos);
-    if (bytes<0) return {{}, 0};
+    const auto bytes = GetUTF8ByteIndex(i->name, pos-i->pos.first_pos.char_index);
+    if (bytes<0) return {};
     const size_t dot_after = i->name.find('.', bytes);
     const size_t limit_len_to = dot_after == string::npos ? i->name.length() : bytes+dot_after;
     //2. Adjust the click pos by adding the num of chars in the prefix not written in the
     //source file to it (If the source file has B.C and we clicked 'C' and the full name
     // of the entity is A.B.C then pos_in_src_in_entity==2 and we now add len('A.') to it.
-    return {std::string_view(i->name).substr(0, prefix_len+limit_len_to),
-            pos_in_src_in_entity + UTF8len(std::string_view(i->name).substr(0, prefix_len))};
+    return { .full_entity_name = std::string_view(i->name).substr(0, prefix_len + limit_len_to),
+            .byte_pos = res.byte_pos + (int)UTF8len(std::string_view(i->name).substr(0, prefix_len)),
+            .csh = this };
 }
 
-std::string_view BlockCsh::AskReplace(std::string_view full_entity, int pos) const {
-    //Take the dot-separated segment 'pos' (as a byte index) is in.
-    //If 'pos' is on a dot, we take what is before.
+std::string_view BlockCsh::AskReplace(std::string_view full_entity_name, int byte_pos) const {
+    //Take the dot-separated segment 'byte_pos' (as a byte index) is in.
+    //If 'byte_pos' is on a dot, we take what is before.
     // E.g., In aaa.bbb.ccc, we take 'aaa' if pos=[0..3], 'bbb' if pos==[4..7], etc.
-    pos = std::max(0, pos);
-    size_t from = full_entity.substr(0, pos).find_last_of('.'); //works even with UTF-8.
+    byte_pos = std::max(0, byte_pos);
+    size_t from = full_entity_name.substr(0, byte_pos).find_last_of('.'); //works even with UTF-8.
     if (from==std::string_view::npos) from = 0;
     else from++;
-    size_t till = full_entity.substr(pos).find_first_of('.');
-    if (till==std::string_view::npos) till = full_entity.length();
-    else till += pos;
-    return full_entity.substr(from, till-from);
+    size_t till = full_entity_name.substr(byte_pos).find_first_of('.');
+    if (till==std::string_view::npos) till = full_entity_name.length();
+    else till += byte_pos;
+    return full_entity_name.substr(from, till-from);
 }
-std::string BlockCsh::ReplaceEntityName(std::string_view full_entity, int pos,
+std::string BlockCsh::ReplaceEntityName(std::string_view full_entity_name, int byte_pos,
                                         std::string_view replace_to,
                                         long lStart, long lEnd) const {
     //Trim block names after the element to replace
-    size_t till = full_entity.substr(pos).find_first_of('.');
-    if (till==std::string_view::npos) till = full_entity.length();
-    else till += pos;
-    full_entity = full_entity.substr(0, till);
+    size_t till = full_entity_name.substr(byte_pos).find_first_of('.');
+    if (till==std::string_view::npos) till = full_entity_name.length();
+    else till += byte_pos;
+    full_entity_name = full_entity_name.substr(0, till);
     std::string text{input_text};
-    const CshPos selection = lStart==lEnd
-        ? CshPos(1, (int)input_text.size())
-        : CshPos(lStart+1, lEnd);
+    if (lStart == lEnd) {
+        lStart = first_pos.char_index;
+        lEnd = INT_MAX;  //We do not care about end_pos here, just something very large
+    }
     //The positions in 'BlockMentions' are in csh units: the first char is
     //indexed 1
     //Collect potential places to replace, where the CSH range
     //- is within the selection
     //- denotes the block we replace (or a child of it)
-    std::vector<std::pair<CshPos, std::string_view>> poses; //these are character indices
+    std::vector<std::pair<CshPos, std::string_view>> poses; 
     for (const auto &[pos, fname, prefix] : BlockMentions)
-        if (std::string_view(fname).substr(0, full_entity.length())== full_entity
-            && selection.IsWithin(pos))
+        if (std::string_view(fname).substr(0, full_entity_name.length()) == full_entity_name
+            && lStart <= pos.first_pos.char_index && pos.end_pos.char_index <= lEnd) {
+            _ASSERT(first_pos.byte_index <= pos.first_pos.byte_index);
+            _ASSERT(pos.end_pos.byte_index <= first_pos.byte_index + (int)input_text.length());
             poses.emplace_back(pos, fname);
+        }
     if (poses.empty()) return text;
-    std::sort(poses.begin(), poses.end(), [](auto &a, auto &b) { return a.first.first_pos<b.first.first_pos; } );
+    std::ranges::reverse(poses); //helps reverse sort
+    std::ranges::sort(poses, std::greater<>()); //sort reverse
     //Find the position of the part to replace in 'full_entity'
     //That is, if we have A.B.C and replace to 'D', we find 'C'.
-    const size_t last_dot = full_entity.find_last_of('.'); //works with UTF8
+    const size_t last_dot = full_entity_name.find_last_of('.'); //works with UTF8
     const size_t replace_pos_in_full_name = last_dot==std::string::npos ? size_t(0) : last_dot+1;
-    const size_t replace_len_in_full_name = full_entity.length()-replace_pos_in_full_name;
+    const size_t replace_len_in_full_name = full_entity_name.length()-replace_pos_in_full_name;
     //Walk the input UTF-8 text to find character positions
-    size_t current_byte = 0; //current byte in the new text
-    size_t current_char = 1; //current char in the old text in CshPos units (starts from 1)
     for (const auto &[pos, fname] : poses) {
-        if (pos.first_pos<=int(current_char)) break;
-        auto bytes = GetUTF8ByteIndex(std::string_view(text).substr(current_byte), pos.first_pos-current_char);
-        if (bytes<0) break;
-        current_byte += bytes;
-        current_char = pos.first_pos;
-        //Find the byte length of the block name in the source text
-        bytes = GetUTF8ByteIndex(std::string_view(text).substr(current_byte), pos.last_pos+1-current_char);
-        if (bytes<0) break;
-        std::string_view src_name = std::string_view(text).substr(current_byte, bytes);
+        std::string_view src_name = input_text.substr(pos.first_pos.byte_index-first_pos.byte_index, pos.byte_length());
         //Locate what part of the text to replace: block name in source (src_name) may be a partial one
         //That is, if we replace A.B.C to D (becoming A.C.D in the process),
         //We may have in the source file (src_name may be):
@@ -1140,13 +1142,8 @@ std::string BlockCsh::ReplaceEntityName(std::string_view full_entity, int pos,
         //Find what position the source text
         _ASSERT(fname.length()>=src_name.length() && fname.substr(fname.length()-src_name.length())==src_name);
         const ptrdiff_t start_at = ptrdiff_t(replace_pos_in_full_name) - ptrdiff_t(fname.length()-src_name.length());
-        if (start_at<0) {
-            current_byte += bytes;
-        } else {
-            text.replace(current_byte+start_at, replace_len_in_full_name, replace_to);
-            current_byte += bytes + replace_to.length() - replace_len_in_full_name;
-        }
-        current_char = pos.last_pos+1;
+        if (0<=start_at)
+            text.replace(pos.first_pos.byte_index+start_at, replace_len_in_full_name, replace_to);
     }
     return text;
 }
