@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -29,7 +29,7 @@ namespace block
 {
 
 /** The line type an arrow between blocks can take.
- * This is used to capture arrow sybols ->*/
+ * This is used to capture arrow symbols ->*/
 enum class EArrowStyle : unsigned char
 {
     SOLID,   ///<Solid line '->'
@@ -55,13 +55,14 @@ enum class EBlockType : unsigned char
 {
     Box,    ///<Classic box
     Boxcol, ///<Classic box with column oriented content
-    Row,    ///<An invisible row of blocks, no marging and no line
+    Row,    ///<An invisible row of blocks, no margin and no line
     Column, ///<An invisible column of blocks, no margin and no line
     Cell,   ///<An invisible set of blocks, continuing the default alignment of the parent, no margin and no line
     Text,   ///<Text with normally no line around it
     Shape,  ///<A block having a specific shape, other than a box
     Space,  ///<Just empty space with a size. Breaks and indicators also have this type.
-    Indicator ///<An indicator
+    Indicator, ///<An indicator
+    Chart   ///<AN inlined chart
 };
 /** Shall we draw tracking rectangles */
 constexpr bool HasArea(EBlockType t) { return t!=EBlockType::Space; }
@@ -101,6 +102,31 @@ constexpr bool IsMin(EDirection t)
 constexpr bool IsMax(EDirection t)
 { return t==EDirection::Below || t==EDirection::Right; }
 
+enum class EOrientation
+{
+    Invalid,     ///<The invalid value. 
+    Normal,      ///<Regular orientation, text is left-to-right.
+    UpsideDown,  ///<The text is upside-down and go from right-to-left.
+    Left,        ///<The text can be viewed from left, text goes down-to-up.
+    Right,       ///<The text can be viewed from right, text goes up-to-down.
+    Tangent,     ///<The text goes along the tangent of the arrow.
+    TangentUpsideDown ///<The text goes along the tangent of the arrow, upside down.
+};
+
+/** True if the orientation is Tangent or TangentUpsideDown*/
+constexpr bool IsTangent(EOrientation o) 
+{ return o==EOrientation::Tangent || o==EOrientation::TangentUpsideDown; }
+
+/** True if the orientation is Normal or Upside Down: if the label reads horizontal.
+ * Note: different from what is horizontal for EDirection.*/
+constexpr bool IsHorizontal(EOrientation t)     {
+ return t== EOrientation::Normal || t== EOrientation::UpsideDown; }
+
+/** True if the direction is Left or Right: if the label reads vertical.
+ * Note: different from what is vertical for EDirection.*/
+constexpr bool IsVertical(EOrientation t) {
+ return t== EOrientation::Left || t== EOrientation::Right; }
+
 struct AlignModifier
 {
     bool major;
@@ -135,8 +161,8 @@ struct EdgePos
  * and thereby its priority in case of conflicts.*/
 enum class EAlignPrio : unsigned char
 {
-    Default,           ///<No aligment was specified, this is the hardcoded default
-    Default_high,      ///<No aligment was specified, this is the hardcoded default + elevated priority
+    Default,           ///<No alignment was specified, this is the hardcoded default
+    Default_high,      ///<No alignment was specified, this is the hardcoded default + elevated priority
     Default_style,     ///<The alignment comes from the default style of the block (including any refinement style)
     Default_style_high,///<The alignment comes from the default style of the block (including any refinement style) + elevated priority
     Content,           ///<The alignment comes from the parent type (coming from the content style of the parent)
@@ -277,7 +303,7 @@ enum class EInternalAlignType : unsigned char
 /** Describes how to internally align content in a block. */
 struct InternalAlignment
 {
-    EInternalAlignType align;    ///<How the children shall be aligned whithin the parent
+    EInternalAlignType align;    ///<How the children shall be aligned within the parent
     EAlignPrio         prio;     ///<The priority of the alignment - to break ties
     FileLineCol        file_pos; ///<The actual position of the attribute
 };
@@ -285,9 +311,10 @@ struct InternalAlignment
 /** A size attribute (for width and height) with a priority
  * And perhaps a list of blocks to take as value.*/
 struct SizeAttr {
-    double size;               ///<Pixel size specified by the user. If 'blocks' is non-empty, it is a multiplier of the size of the bounding box of those blocks.
-    EAlignPrio prio;           ///<The priority of this attribute (stye, explicit, etc.)
-    StringWithPosList blocks{};///<If this is non-empty, use the specific double value as a multiplier
+    double size;                      ///<Pixel size specified by the user. If 'blocks' is non-empty, it is a multiplier of the size of the bounding box of those blocks.
+    EAlignPrio prio;                  ///<The priority of this attribute (style, explicit, etc.)
+    bool other_dim_of_blocks = false; ///<If set than the size specified here (e.g., for width) shall be taken from othe other dimension (that is, height) of the blocks listed.
+    StringWithPosList blocks = {};    ///<If this is non-empty, use the specific double value as a multiplier
 };
 
 /** An optional size attribute (for width and height) with a priority */
@@ -308,8 +335,9 @@ enum class EArrowRouting
     Invalid,
     Horizontal,///<Strictly horizontal
     Vertical,  ///<Strictly vertical
+    Grid,      ///<Either Horizontal or Vertical
     Straight,  ///<Straight, disregard any blocks in the way
-    Polygon,   ///<Polygon with potential rounded or bevelled corners
+    Polygon,   ///<Polygon with potential rounded or beveled corners
     Curvy,     ///<Smooth line
     Manhattan  ///<Only 90 degree edges
 };
@@ -319,23 +347,32 @@ enum class EArrowRouting
 enum class EBlockStyleType
 {
     Unspecified, ///<No change, or not relevant
-    Generic,     ///<This is a generic style defined by the user that can be appled here or there.
-    Content,     ///<This is a content style appled as a default to the running style
+    Generic,     ///<This is a generic style defined by the user that can be applied here or there.
+    Content,     ///<This is a content style applied as a default to the running style
     Running      ///<This is a running style
 };
 
+/** Specifies if the style applies to a block or an arrow*/
+enum class EBlockStyleTarget
+{
+    Any,
+    Block,
+    Arrow
+};
+
 /** Style for Block charts.
  * Note that we use EStyleType::OPTION for the running style. */
 class BlockStyle : public SimpleStyleWithArrow<TwoArrowHeads>
 {
     bool f_label;           ///<Flag specifying if this style has labels, label.pos, label.align, label.orient
     bool f_alignment;       ///<Flag specifying if this style has alignment attributes "top", "bottom" etc,
-    bool f_content;         ///<Flag specifying if this style has attributes governing children alignemnt "content.*" and "allow_routing" and "indicator"
+    bool f_content;         ///<Flag specifying if this style has attributes governing children alignment "content.*" and "allow_routing" and "indicator"
     bool f_margin;          ///<Flag specifying if this style has margin attributes "margin.*" "imargin.*", "width", "height", "size" and "multi.*"
     bool f_routing;         ///<Flag specifying if this style has 'routing' attribute
     bool f_marker;          ///<Flag specifying if this style has 'marker' attributes
 public:
     EBlockStyleType            block_style_type;      ///<If type==EStyleType::STYLE, this specifies exactly what kind of style we are. Should be set to Unspecified if type!=STYLE
+    EBlockStyleTarget          block_style_target;    ///<What kind of element the style applies to
     OptAttrLine<std::string>   label;                 ///<The label of this element
     AlignmentAttr              alignment;             ///<Our alignment
     OptAttr<InternalAlignment> XChildAlign;           ///<Describes how our content shall be aligned within us horizontally
@@ -347,11 +384,12 @@ public:
     OptAttr<double>            max_imargin[2];        ///<Margin on the left ([0]) and top ([1]) side inside the block
     OptAttr<bool>              allow_arrows;          ///<True if we allow arrows to cross us
     OptSizeAttr                size[2];               ///<Specific width ([0]) and height ([1])
+    OptAttr<double>            aspect_ratio;          ///<Specifies the aspect ratio: width/height. E.g., 3:2 will be 1.5.
     OptAttrLine<ELabelMode>    label_mode;            ///<How to interpret the size attribute for label-only blocks
     OptAttr<double>            multi_offset[2];       ///<Specifies what offset to apply with the "multi" keyword
     OptAttrLine<EDirection>    label_pos;             ///<Position of the label of a box relative to its content
     OptAttr<double>            label_align;           ///<left/center/right alignment of label
-    OptAttr<EDirection>        label_orient;          ///<How to orient the label. Above is normal...
+    OptAttr<EOrientation>      label_orient;          ///<How to orient the label. Above is normal...
     OptAttr<EArrowRouting>     routing;               ///<Routing method of an arrow
     OptAttr<StringWithPosList> route_cross;           ///<Blocks arrows can freely route across
     OptAttr<StringWithPosList> route_cross_parent;    ///<Blocks arrows can freely route across, including their children
@@ -359,20 +397,22 @@ public:
     OptAttr<StringWithPosNumberList>
                                distance_per_block;    ///<Distance to be kept by arrows from specific blocks
     OptAttr<double>            route_factor;          ///<The strategy of selecting between paths. 0=length only, 1=turns only.
-    OptAttr<double>            route_arrow_distance;  ///<The minimum distance to keep to other arrows. If zero, dont de-overlap this arrow.
+    OptAttr<double>            route_arrow_distance;  ///<The minimum distance to keep to other arrows. If zero, don't de-overlap this arrow.
     OptAttr<bool>              route_block_others;    ///<If true, we add our path to blocking arrows laid out subsequent to us in the same de-overlap group. Only if we were de-overlapped
     OptAttr<bool>              route_linearextend;    ///<If true, we end at a block, get de-overlapped and our new end is not inside the block, but it was before the de-overlapping and our linear extension crosses the block - we linearextend us up until the block.
     OptAttr<bool>              route_clip_block;      ///<If true, any arrow at the end of the block clips the arrow & we do de-overlapping.
+    OptAttr<bool>              routing_try_harder;    ///<If true, We do an exhaustive search on best layout at de-overlapping.
+    OptAttr<bool>              routing_allow_joint_layout; ///<If true, we allow laying out jointly with other arrows if the initial layout was equally the same.
     OneArrowHead               marker;                ///<If true and we have de-overlapped this arrow its path will be added to the blocking parts of its peers.
 	StringWithPosList          draw_time;             ///<If not empty, the value of the 'draw_before' or 'draw_after' attribute(s). A block(s), we need to be drawn before/after.
 	bool                       draw_time_is_before;   ///<Tells us if 'draw_time' is before or after
     OptAttr<bool>              indicator;             ///<If collapsed blocks show an indicator or not
 
-    BlockStyle(EStyleType tt, EBlockStyleType btt, EColorMeaning cm, bool a,
+    BlockStyle(EStyleType tt, EBlockStyleType btt, EBlockStyleTarget ttgt, EColorMeaning cm, bool a,
                bool t, bool l, bool f, bool s, bool nu, bool shp,
                bool lab, bool al, bool c, bool m, bool r, bool mkr); ///Control which components are used
-    explicit BlockStyle(EStyleType tt, EColorMeaning cm = EColorMeaning::NOHOW); //Has all the components, but is empty
-    explicit BlockStyle(EBlockStyleType btt = EBlockStyleType::Generic, EColorMeaning cm = EColorMeaning::NOHOW); //Has all the components, but is empty
+    explicit BlockStyle(EStyleType tt, EBlockStyleTarget ttgt = EBlockStyleTarget::Any, EColorMeaning cm = EColorMeaning::NOHOW); //Has all the components, but is empty
+    explicit BlockStyle(EBlockStyleType btt = EBlockStyleType::Generic, EBlockStyleTarget ttgt = EBlockStyleTarget::Any, EColorMeaning cm = EColorMeaning::NOHOW); //Has all the components, but is empty
     BlockStyle(const BlockStyle&) = default;
     BlockStyle(BlockStyle&&) = default;
     BlockStyle &operator=(const BlockStyle&) = default;
@@ -402,7 +442,7 @@ struct Parent : std::variant<const BlockBlock*, std::string> {
     std::string_view GetName() const;
 };
 
-/** Context for Graphs. Just a styleset and colors, really.*/
+/** Context for Graphs. Just a style set and colors, really.*/
 class BlockContext : public ContextBase<BlockStyle>
 {
 public:
@@ -422,9 +462,9 @@ public:
      * @param [in] l The first character of the context in the input file.*/
     BlockContext(bool f, EContextParse p, EContextCreate t, const FileLineCol &l) :
         ContextBase<BlockStyle>(f, p, EContextCreate::CLEAR, l),
-        running_style_blocks(std::make_unique<BlockStyle>(EStyleType::STYLE, EBlockStyleType::Running, EColorMeaning::FILL, false,
+        running_style_blocks(std::make_unique<BlockStyle>(EStyleType::STYLE, EBlockStyleType::Running, EBlockStyleTarget::Block, EColorMeaning::FILL, false,
                                                           true, true, true, true, true, true, true, true, true, true, false, false)), //no arrow, routing, marker
-        running_style_arrows(std::make_unique<BlockStyle>(EStyleType::STYLE, EBlockStyleType::Running, EColorMeaning::LINE_ARROW_TEXT, true,
+        running_style_arrows(std::make_unique<BlockStyle>(EStyleType::STYLE, EBlockStyleType::Running, EBlockStyleTarget::Arrow, EColorMeaning::LINE_ARROW_TEXT, true,
                                                           true, true, false, false, true, false, true, false, false, false, true, true)) //just arrow, text, line and number, label, routing and marker
     { switch (t) {
         default: _ASSERT(0); FALLTHROUGH;
