@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -990,10 +990,10 @@ template<bool y>
 std::array<std::pair<XY, double>,2> Edge::Extremes() const
 {
     if (IsStraight())
-        return (start()[y]<end()[y]) ?
-        std::array<std::pair<XY, double>, 2>{std::pair<XY, double>{start(), 0.}, std::pair<XY, double>{end(), 1.}} :
-        std::array<std::pair<XY, double>, 2>{std::pair<XY, double>{ end(), 1. }, std::pair<XY, double>{start(), 0.}};
-
+        return start()[y]<end()[y]
+             ? std::array<std::pair<XY, double>, 2>{std::pair<XY, double>{start(), 0.}, std::pair<XY, double>{end(), 1.}}
+             : std::array<std::pair<XY, double>, 2>{std::pair<XY, double>{ end(), 1. }, std::pair<XY, double>{start(), 0.}};
+    
     //from: http://stackoverflow.com/questions/2587751/an-algorithm-to-find-bounding-box-of-closed-bezier-curves
     std::vector<double> pos;
     std::vector<XY> xy;
@@ -1229,8 +1229,8 @@ inline bool Edge::RemoveLoops(std::vector<E> &edges, bool self) {
  * @param [out] pos_other The pos values of the corresponding crosspoints on the other edge.
  * @returns The number of crosspoints found [0..MAX_CP]
  */
-inline unsigned Edge::Crossing(const Edge& A, bool is_next, XY r[Edge::MAX_CP], double pos_my[Edge::MAX_CP], double pos_other[Edge::MAX_CP]) const {
-    return abs(CrossingWithOverlap(A, is_next, r, pos_my, pos_other));
+inline unsigned Edge::Crossing(const Edge& A, bool is_next, XY r[Edge::MAX_CP], double pos_my[Edge::MAX_CP], double pos_other[Edge::MAX_CP]) const {    
+    return HullOverlap(A, is_next) ? abs(CrossingWithOverlap(A, is_next, r, pos_my, pos_other)) : 0;
 }
 
 /** Calculates the crosspoints with another edge.
@@ -1251,7 +1251,9 @@ inline unsigned Edge::Crossing(const Edge& A, bool is_next, XY r[Edge::MAX_CP],
 inline EdgeCPsDataWithXY<Edge::MAX_CP> Edge::Crossing(const Edge &A, bool is_next, bool *overlap) const
 {
     EdgeCPsDataWithXY<Edge::MAX_CP> ret;
-    const int n = CrossingWithOverlap(A, is_next, ret.xy.data(), ret.pos_my.data(), ret.pos_other.data());
+    const int n = HullOverlap(A, is_next) 
+        ? CrossingWithOverlap(A, is_next, ret.xy.data(), ret.pos_my.data(), ret.pos_other.data())
+        : 0;
     ret.num = abs(n);
     if (overlap) *overlap = n<0;
     return ret;
@@ -1510,7 +1512,102 @@ void Edge::TangentFrom(const Edge &o, XY clockwise[2], XY cclockwise[2]) const
 }
 
 
-} //namespace contour bezier
+/** Checks if the bounding box of the hull of two overlap.
+ * @returns True if there is overlap */
+inline bool HullOverlap(const Block& T, const Block &O, bool is_next) {
+    //We allow hulls a bit apart to be reported as overlapping.
+    //This is after all just to exclude edges far apart
+    //was: if (!O.Overlaps(T)) return false;
+    if (test_smaller(T.y.till, O.y.from) || test_smaller(O.y.till, T.y.from)) return false;
+    if (test_smaller(T.x.till, O.x.from) || test_smaller(O.x.till, T.x.from)) return false;
+    //test if 'o' and 'this' only overlaps at their start/endpoint
+    if (!is_next) return true;
+    if (O.y.from != T.y.till && O.y.till != T.y.from)
+        return true;
+    if (O.x.from != T.x.till && O.x.till != T.x.from)
+        return true;
+    //OK, now we know that the two BBs touch only at a single vertex
+    //since end==o.start, this can only be that point
+    return false;
+}
+
+
+/** Checks if the bounding box of the hull of two overlap.
+ * We place it here for possible inlining as this is called very often.
+ * @param [in] o The other edge
+ * @param [in] is_next If true 'o' is an edge following 'this' and we ignore if they
+ *             connect via this->end == o.start.
+ * @returns True if there is overlap */
+inline bool Edge::HullOverlap(const Edge& o, bool is_next) const {
+    const Block O = o.IsStraight() ? Block(o.start(), o.end()) : o.GetBezierHullBlock();
+    const Block T = IsStraight() ? Block(start(), end()) : GetBezierHullBlock();
+    return contour::HullOverlap(T, O, is_next && end()==o.start());
+}
+
+
+/** Calculates the tight bounding box of 'this'.
+ * Moderately expensive for beziers. We place it to the
+ * header to be able to get inlined, which would make 
+ * straight edges much faster.*/
+inline Block Edge::CreateBoundingBox() const noexcept
+{
+    if (IsStraight())
+        return Block(start(), end());
+    //from: http://stackoverflow.com/questions/2587751/an-algorithm-to-find-bounding-box-of-closed-bezier-curves
+    std::array<XY, 6> bounds = {start(), end()};
+    unsigned p = 2;
+
+    for (unsigned i = 0; i < 2; ++i) {
+        double b = 6 * start()[i] - 12 * c1()[i] + 6 * c2()[i];
+        double a = -3 * start()[i] + 9 * c1()[i] - 9 * c2()[i] + 3 * end()[i];
+        double c = 3 * c1()[i] - 3 * start()[i];
+        if (fabs(a) < 1e-8) {
+            if (fabs(b) < 1e-8) {
+                continue;
+            }
+            double t = -c / b;
+            if (0 < t && t < 1)
+                bounds[p++] = Split(t);
+            continue;
+        }
+        double b2ac = pow(b, 2) - 4 * c * a;
+        if (b2ac < 0) {
+            continue;
+        }
+        double  t1 = (-b + sqrt(b2ac))/(2 * a);
+        if (0 < t1 && t1 < 1)
+            bounds[p++] = Split(t1);
+        double  t2 = (-b - sqrt(b2ac))/(2 * a);
+        if (0 < t2 && t2 < 1)
+            bounds[p++] = Split(t2);
+    }
+    const auto X = std::minmax_element(bounds.begin(), bounds.begin()+p,
+                     [](const XY &a, const XY &b) {return a.x<b.x; });
+    const auto Y = std::minmax_element(bounds.begin(), bounds.begin()+p,
+                     [](const XY &a, const XY &b) {return a.y<b.y; });
+    return Block(X.first->x, X.second->x, Y.first->y, Y.second->y);
+}
+
+inline double Edge::GetAreaAboveAdjusted() const noexcept {
+    if (IsStraight())
+        return start().x * end().y - start().y * end().x;
+    Edge E1, E2;
+    Split(E1, E2);
+    return E1.GetAreaAboveAdjusted() + E2.GetAreaAboveAdjusted();
+}
+
+inline double Edge::GetLength() const noexcept {
+    if (IsStraight())
+        return (start() - end()).length();
+    Edge E1, E2;
+    Split(E1, E2);
+    return E1.GetLength() + E2.GetLength();
+}
+
+
+
+
+} //namespace contour 
 
 
 
