@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -23,38 +23,33 @@
 
 #define _CRT_NONSTDC_NO_DEPRECATE
 
-#include <cassert>
 #include <algorithm>
 #include <map>
-#include <set>
-#include <stack>
 #include <cmath>
 #include "contour.h"
 #include "contour_debug.h"
 
 namespace contour {
 
-
-
-
 //Do not create degenerate triangles.
 //Always create clockwise.
-SimpleContour::SimpleContour(XY a, XY b, XY c) :
-    clockwise_fresh(true), boundingBox_fresh(true), area_fresh(false),
-    clockwise(true), boundingBox(a, b) {
+SimpleContour::SimpleContour(XY a, XY b, XY c) : boundingBox(a, b) {
     switch (triangle_dir(a, b, c)) {
     default: //create empty if degenerate triangle
         boundingBox.MakeInvalid();
+        area = 0;
         return;
     case CLOCKWISE:
         edges.emplace_back(a, b);
         edges.emplace_back(b, c);
         edges.emplace_back(c, a);
+        area = maxa;
         break;
     case COUNTERCLOCKWISE:
         edges.emplace_back(a, c);
         edges.emplace_back(c, b);
         edges.emplace_back(b, a);
+        area = maxa; //we make the triangle clockwise, so indicate that
         break;
     }
     boundingBox += c;
@@ -63,30 +58,28 @@ SimpleContour::SimpleContour(XY a, XY b, XY c) :
 /** Create an ellipse (or ellipse slice) shape.
  * If not a full ellipse/circle a line segment is added as _the last edge_ to close it.
  *
- * @param [in] c Centerpoint
+ * @param [in] c Center point
  * @param [in] radius_x Radius in the y direction. Its absolute value is used.
  * @param [in] radius_y Radius in the y direction (same as `radius_x` if omitted = circle) Its absolute value is used.
  * @param [in] tilt_deg The tilt of the ellipse in degrees. 0 if omitted.
- * @param [in] s_deg The startpoint of the arc.
+ * @param [in] s_deg The start point of the arc.
  * @param [in] d_deg The endpoint of the arc. If equal to `s_deg` a full ellipse results, else a straight line is added to close the arc.
  * If `radius_x` is zero, we return an empty shape. If `radius_y` is zero, we assume it to be the same as `radius_x` (circle).
 */
 SimpleContour::SimpleContour(const XY& c, double radius_x, double radius_y, double tilt_deg, double s_deg, double d_deg) :
-    edges(c, radius_x, radius_y, tilt_deg, s_deg, d_deg, true, true),
-    clockwise_fresh(true), boundingBox_fresh(false), area_fresh(false),
-    clockwise(true) {
+    area(maxa), edges(c, radius_x, radius_y, tilt_deg, s_deg, d_deg, true, true)
+{
     if (size()==0) {
-        area_fresh = true;
-        boundingBox_fresh = true;
         area = 0;
         boundingBox.MakeInvalid();
         return;
-    }
+    } else
+        dirty_bb();
 }
 
 /** Clear the shape and assign a rectangle shape. */
 SimpleContour& SimpleContour::operator =(const Block& b) {
-    clear(); //clockwise is set to fresh and true here, bounding box & area is set to fresh
+    clear(); //empty bb and zero area
     if (b.IsInvalid() || !test_smaller(b.x.from, b.x.till) || !test_smaller(b.y.from, b.y.till)) return *this;
     edges.push_back(Edge(b.UpperLeft(), XY(b.x.till, b.y.from)));
     edges.push_back(Edge(XY(b.x.till, b.y.from), b.LowerRight()));
@@ -103,8 +96,7 @@ SimpleContour& SimpleContour::Invert() noexcept {
         at(i).Invert();
     for (size_t i = 0; i<size()/2; i++)
         std::swap(at(i), at(size()-1-i));
-    area *= -1;
-    clockwise = !clockwise;
+    if (area!=infa) area *= -1;
     return *this;
 }
 
@@ -114,7 +106,7 @@ SimpleContour& SimpleContour::Invert() noexcept {
  * @param [out] edge If not nullptr it can receive the number of the edge if `p` lies on an edge/vertex.
  * @param [out] pos If not nullptr it can receive the position of `p` on the edge if `p` lies on one.
  * @param [in] strict If false, points close to edges will be reported as on the edge.
- * @returns The relation. Note that for counerclockwise 'inside' is the same as if the
+ * @returns The relation. Note that for counterclockwise 'inside' is the same as if the
             contour were clockwise, that is in the limited space enclosed.
  */
 EPointRelationType SimpleContour::IsWithin(XY p, size_t* edge, double* pos, bool strict) const noexcept {
@@ -127,7 +119,7 @@ EPointRelationType SimpleContour::IsWithin(XY p, size_t* edge, double* pos, bool
     //2. a rightward edge excludes its starting endpoint, and includes its final endpoint;
     //3. vertical edges are excluded; and
     //4. the edge-ray intersection point must be strictly below of the point P.
-    //4b: since we say containment also for edge points, if the egde goes through p, we stop
+    //4b: since we say containment also for edge points, if the edge goes through p, we stop
     size_t count = 0;
     size_t e;
     for (e = 0; e<size(); e++) {
@@ -174,7 +166,7 @@ EPointRelationType SimpleContour::IsWithin(XY p, size_t* edge, double* pos, bool
 }
 
 
-//////////////////////////////////SimpleContour::Combine (Union, intersection, subsrtaction) implementation
+//////////////////////////////////SimpleContour::Combine (Union, intersection, substation) implementation
 
 
 /** Test if q is on the a->b section.
@@ -229,13 +221,13 @@ EContourRelationType SimpleContour::CheckContainmentHelper(const SimpleContour&
             case WI_INSIDE:
                 if (bb_rel==WI_ON_EDGE || bb_rel==WI_ON_VERTEX)
                     //This is clearly a bad result as the point cannot be inside,
-                    //if it is on the preimiter of the bouinding box. Try another.
+                    //if it is on the perimeter of the bounding box. Try another.
                     continue;
                 inside++;
             check_vertical:
                 //If the previous edge was vertical, then here we were checking
                 //being in/out using the same X coordinate. If there were some imprecision
-                //before, we may have that, too, so inrease the threshold, to force us to check one more.
+                //before, we may have that, too, so increase the threshold, to force us to check one more.
                 if (i>0 && threshold<size() && test_equal(at(i-1).GetStart().x, p.x))
                     threshold++;
                 break;
@@ -253,7 +245,7 @@ EContourRelationType SimpleContour::CheckContainmentHelper(const SimpleContour&
                 RayAngle next1 = at(i).Angle(false, 0);
                 //Normalize so that the clockwise next->prev angle transition is the inside of the
                 //space enclosed by the paths, irrespective of whether the direction of the
-                //simplecontours is clockwise or clounterclockwise.
+                //simplecontours is clockwise or counterclockwise.
                 if (!GetClockWise()) std::swap(prev1, next1); //make angles as if clockwise
                 if (!b.GetClockWise()) std::swap(prev2, next2); //make angles as if clockwise
 
@@ -337,36 +329,38 @@ EContourRelationType SimpleContour::CheckContainment(const SimpleContour &other)
 /** Swap data with `b`. */
 void SimpleContour::swap(SimpleContour& b) noexcept
 {
-    edges.swap(b.edges);
     std::swap(boundingBox, b.boundingBox);
     std::swap(area, b.area);
-    //std::swap() does not work on bitfields, so we do it as below.
-    bool B;
-    B = clockwise; clockwise = b.clockwise; b.clockwise = B;
-    B = clockwise_fresh; clockwise_fresh = b.clockwise_fresh; b.clockwise_fresh = B;
-    B = area_fresh; area_fresh = b.area_fresh; b.area_fresh = B;
-    B = boundingBox_fresh; boundingBox_fresh = b.boundingBox_fresh; b.boundingBox_fresh = B;
+    edges.swap(b.edges);
 }
 
 
 void SimpleContour::assign_dont_check(std::span<const XY> v)
 {
-    clear();
-    if (v.size()<2) return;
+    if (v.size() < 2) {
+        clear();
+        return;
+    }
+    dirty_bb();
+    area = infa;
+    edges.clear();
     for (size_t i = 0; i<v.size(); i++)
         edges.push_back(Edge(v[i], v[(i+1)%v.size()]));
-    clockwise_fresh = boundingBox_fresh = area_fresh = false;
     Sanitize();  //includes CalculateBoundingBox() and CalculateClockwise()
 }
 
 void SimpleContour::assign_dont_check(std::initializer_list<XY> v)
 {
-    clear();
-    if (v.size()<2) return;
+    if (v.size() < 2) {
+        clear();
+        return;
+    }
+    dirty_bb();
+    area = infa;
+    edges.clear();
     for (auto i= v.begin(), j=std::next(i); j!=v.end(); ++i, ++j)
         edges.emplace_back(*i, *j);
     edges.emplace_back(*std::prev(v.end()), *v.begin());
-    clockwise_fresh = boundingBox_fresh = area_fresh = false;
     Sanitize();  //includes CalculateBoundingBox() and CalculateClockwise()
 }
 
@@ -407,7 +401,7 @@ bool SimpleContour::Sanitize(double tolerance)
                 goto clear;
             else at(u).SetEndOnly(at_next(u).GetStart());
     }
-    if (!ret) area_fresh = boundingBox_fresh = false; //we think clockwiseness remains just by deleting a few edges.
+    if (!ret) { dirty_bb(); dirty_area(); } //we think clockwiseness remains just by deleting a few edges.
     return ret;
 clear:
     clear();
@@ -424,7 +418,6 @@ const Block &SimpleContour::CalculateBoundingBox() const noexcept
             boundingBox += e.GetStart();
         else
             boundingBox += e.CreateBoundingBox();
-    boundingBox_fresh = true;
     return boundingBox;
 }
 
@@ -471,36 +464,12 @@ bool SimpleContour::AddAnEdge(const Edge &edge)
             if (ret[i].Crossing(ret[j], next(i)==j, dum1, dum2, dum2))
                 return false;
     //OK, we can have these edges inserted
+    dirty_bb();
+    area = infa;
     swap(ret);
-    area_fresh = boundingBox_fresh = clockwise_fresh = false; //clockwiseness could change by adding a curvy edge
     return true;
 }
 
-/** Calculate & cache the surface area of the shape.
- *
- * This is based on <http://www.mathopenref.com/coordpolygonarea.html>
- * For each edge, we calculate the area above it (the area between the edge and Y axis,
- * but since y grows downward, the area is "above" the edge).
- * The value is signed. If the edge is from right to left it is positive, else negative.
- * Since (for straight edges) the area would contain "start.x*start.y-end.x*end.y",
- * which will cancel out. Plus it is easier to return twice the area.
- * Thus we request edge to return
- * "2*(area_above_edge - start.x*start.y + end.x*end.y)", this is what
- * Edge::GetAreaAboveAdjusted() returns.
- *
- * After this function the area cache will hold this value for instant access.
- *
- * @returns the calculated area. Negative if shape is counterclockwise.
- */
-double SimpleContour::CalculateArea() const noexcept
-{
-    double ret = 0;
-    for (const auto &e: edges)
-        ret += e.GetAreaAboveAdjusted();
-    area_fresh= true;
-    return area = ret/2;
-}
-
 /** Calculate the circumference length of the shape.
  *
  * @param [in] include_hidden If false, we leave out edges marked as non-visible.
@@ -614,7 +583,7 @@ void SimpleContour::CreateRoundForExpand(XY center, const XY &start, const XY &e
     //But, if we use the clockwiseness of the original contour, we are safe
     //since for positive originals we will keep only positive results,
     //so if we miss and we will be part of a negative contour, that
-    //will get dropped anyway. (Likewise for negative orgiginals.)
+    //will get dropped anyway. (Likewise for negative originals.)
     SimpleContour round(center, radius, radius, 0, clockwise ? s_deg : d_deg, clockwise ? d_deg : s_deg);
     //delete the straight edge
     if (round.edges.back().IsStraight())
@@ -937,8 +906,8 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
                 //  of the first edge (in "i")
                 //- if both of the edges are straight, the cp is as for the two bezier cases.
                 //  This can happen if the last segment of a bezier has turned to a line during expansion.
-                //NOTE that FindExpandedEdgesCP() uses the tangens of the *original* edge, thus the cp
-                //is in the direction of the origial edge ends even if a bezier edge changed direction
+                //NOTE that FindExpandedEdgesCP() uses the tangents of the *original* edge, thus the cp
+                //is in the direction of the original edge ends even if a bezier edge changed direction
                 //during expansion.
 
 
@@ -952,7 +921,7 @@ SimpleContour::MetaDataList SimpleContour::ExpandHelper(EExpandType type, double
 
                 //If we are at the end of the list (and next_i & end_orig_next
                 //are at the beginning, we copy them to the end, so RemoveLoop
-                //can operate on a contigous series of edges.
+                //can operate on a contiguous series of edges.
                 //(RemoveLoop does not do circular iterator increments/decrements)
                 if (done) {
                     r.splice(r.end(), r, next_i, end_orig_next);
@@ -1094,7 +1063,9 @@ bool SimpleContour::IsConcaveJoin(size_t u) const
     //test if we cross each other
     XY r[9];
     double pos_my[9], pos_other[9];
-    const int num = at(u).CrossingWithOverlap(at_next(u), true, r, pos_my, pos_other); //num<0 if the two edges overlap
+    const int num = at(u).HullOverlap(at_next(u), true)
+        ? at(u).CrossingWithOverlap(at_next(u), true, r, pos_my, pos_other) //num<0 if the two edges overlap
+        : 0;
     if (num<0) return true; //overlapping edges are concave by default.
     if (num) {
         //if we have crosspoints, take the one closes to the vertex, halven the positions
@@ -1137,26 +1108,26 @@ bool SimpleContour::IsConcaveJoin(size_t u) const
  * @returns The expanded shape (completely untangled)*/
 Contour SimpleContour::CreateExpand(EExpandType type, double gap, double miter_limit) const
 {
-    Contour ret;
     if (gap==0 || size()==0)
-        return *this; //no change
+        return Contour{ *this }; //no change
     MetaDataList r = ExpandHelper(type, gap, miter_limit, 0.01);
-    if (r.size()==0) return ret; //empty shape
-    SimpleContour sp_r;
-    sp_r.edges.reserve(r.size());
+    if (r.size() == 0) return {}; //empty shape
+    Contour sp_r;
+    sp_r.first.outline.dirty_bb();
+    sp_r.first.outline.area = infa;
+    sp_r.first.outline.edges.reserve(r.size());
     for (auto &e : r)
-        sp_r.edges.push_back(e);
-    sp_r.clockwise_fresh = sp_r.area_fresh = sp_r.boundingBox_fresh = false;
+        sp_r.first.outline.edges.push_back(e);
 
 #ifdef _DEBUG
-    SimpleContour sp_r2;
-    sp_r2.edges = sp_r.GetEdges();
-    sp_r2.clockwise_fresh = sp_r2.area_fresh = sp_r2.boundingBox_fresh = false;
+    SimpleContour sp_r2 = SimpleContour::UnsafeMake(Path(sp_r.first.outline.GetEdges())); //take a copy
 #endif
 
-    sp_r.Sanitize(EXPAND_TOLERANCE); //remove degenerate or very short edges
-    //OK, now untangle
-    ret.Operation(GetClockWise() ? Contour::EXPAND_POSITIVE : Contour::EXPAND_NEGATIVE, Contour(std::move(sp_r)));
+    sp_r.first.outline.Sanitize(EXPAND_TOLERANCE); //remove degenerate or very short edges
+    //OK, now untangle, this does not need a bounding box, avoid calculating one.
+    Contour ret;
+    ret.Operation(GetClockWise() ? Contour::EXPAND_POSITIVE : Contour::EXPAND_NEGATIVE, std::move(sp_r));
+    _ASSERT(ret.IsEmpty() || !ret.GetBoundingBox().IsInvalid());
 
 #ifdef _DEBUG
     if (0) {
@@ -1229,8 +1200,8 @@ Contour SimpleContour::CreateExpand2D(const XY &gap) const
 {
     //if we do nothing, we return us
     if (gap.x==0 && gap.y==0)
-        return  *this;
-    //if we srink too much we return empty
+        return Contour{ *this };
+    //if we shrink too much we return empty
     if ((GetBoundingBox().x.Spans()<-2*gap.x && GetClockWise()) ||
         (GetBoundingBox().y.Spans()<-2*gap.y && GetClockWise()) ||
         (GetBoundingBox().x.Spans()<2*gap.x && !GetClockWise()) ||
@@ -1253,13 +1224,14 @@ Contour SimpleContour::CreateExpand2D(const XY &gap) const
 
     _ASSERT(r2.first.outline.edges.size()==0 || r2.first.outline.edges.front().GetStart()==r2.first.outline.edges.back().GetEnd());
 
-    r2.first.outline.clockwise_fresh = r2.first.outline.area_fresh = r2.first.outline.boundingBox_fresh = false;
-    r2.first.outline.Sanitize(); //calculates clockwise and boundingbox, as well
+    r2.first.outline.Sanitize(); //remove degenerate or very short edges, make them connect
     if (r2.first.outline.size()==0) return Contour();
-    r2.boundingBox = r2.first.outline.GetBoundingBox(); //copy bb to outer
-    //OK, now untangle
+    r2.first.outline.dirty_bb();
+    r2.first.outline.area = infa;
+   //OK, now untangle, this does not need a bounding box, avoid calculating one.
     Contour res;
     res.Operation(GetClockWise() ? Contour::EXPAND_POSITIVE : Contour::EXPAND_NEGATIVE, std::move(r2));
+    _ASSERT(res.IsEmpty() || !res.GetBoundingBox().IsInvalid());
     //XXX Fix this better:
     if ((gap.x<0) == GetClockWise()) {
         //Kill those cwh's which are not entirely inside the original
@@ -1288,7 +1260,7 @@ EContourRelationType SimpleContour::RelationTo(const SimpleContour &c) const
     if (std::ranges::any_of(cps, [](const CPData& d) { return d.type.rel==CPRel::Cross; }))
         return REL_OVERLAP;
     //Perhaps here we could evaluate the touching crosspoints to see relation for touching
-    //contours as an optimization. Left as excercise to the reader for now.
+    //contours as an optimization. Left as exercise to the reader for now.
     //The two shapes are apart or one inside the other, safe to call this
     return CheckContainment(c);
 }
@@ -1296,7 +1268,7 @@ EContourRelationType SimpleContour::RelationTo(const SimpleContour &c) const
 /** Determine the relative vertical distance between two shapes.
  *
  * This is a helper for OffsetBelow, that is called if the x range of bounding boxes
- * actually overlap and there is a chance of resuling in an offset less than infinity.
+ * actually overlap and there is a chance of resulting in an offset less than infinity.
  */
 double SimpleContour::do_offsetbelow(const SimpleContour &below, double &touchpoint, double offset) const
 {
@@ -1325,8 +1297,8 @@ void SimpleContour::VerticalCrossSection(double x, DoubleMap<bool> &section) con
     }
 }
 
-/** Creates a 'gap' wide mesh gradient at our perimiter, but completely within us.
-  * 'gap' is the width of the graient, {r,g,b,a}1 is the inner color, {r,g,b,a}2 is the outer.
+/** Creates a 'gap' wide mesh gradient at our perimeter, but completely within us.
+  * 'gap' is the width of the gradient, {r,g,b,a}1 is the inner color, {r,g,b,a}2 is the outer.
   * If shrinking us makes us fall into pieces, we return nullptr as sign of failure.
   * We also return the inner edge of the mesh, so that it can be filled solid.*/
 std::pair<cairo_pattern_t *, Contour>
@@ -1336,8 +1308,8 @@ SimpleContour::CairoMeshGradient(double gap, double r1, double g1, double b1, do
 {
     _ASSERT(gap>0);
     std::pair<cairo_pattern_t *, Contour> ret = { nullptr, CreateExpand(type, -gap, miter_limit) };
-    //if both original shape and its shrunken version is a single contigous contour,
-    //we can do a mesh gradent shadow
+    //if both original shape and its shrunken version is a single contiguous contour,
+    //we can do a mesh gradient shadow
     if (ret.second.size()!=1 || ret.second[0].HasHoles())
         return ret;
     cairo_pattern_t *pattern = cairo_pattern_create_mesh();
@@ -1486,8 +1458,8 @@ std::array<CPData, 2> SimpleContour::CutEx(const Edge &s) const
 bool SimpleContour::CrossPoints(std::vector<CPData> &ret, const Edge &e, CPTasks to_do, bool swap) const
 {
     const size_t orig_ret_size = ret.size();
-    bool r = edges.CrossPoints(ret, true, boundingBox_fresh ? &boundingBox : nullptr,
-                               &e, &e+1, false, boundingBox_fresh ? &static_cast<const Block &>(e.CreateBoundingBox()) : nullptr, to_do, swap);
+    bool r = edges.CrossPoints(ret, true, is_bb_dirty() ? nullptr : &boundingBox,
+                               &e, &e+1, false, is_bb_dirty() ? nullptr : &static_cast<const Block &>(e.CreateBoundingBox()), to_do, swap);
     for (size_t i = orig_ret_size; i<ret.size(); i++)
         (swap ? ret[i].contour_other : ret[i].contour_me) = this;
     return r;
@@ -1495,7 +1467,7 @@ bool SimpleContour::CrossPoints(std::vector<CPData> &ret, const Edge &e, CPTasks
 
 
 /** Chop the Contour into paths at points. We return an empty vector if
- * a pathpos is invalid or you give us only one.
+ * a PathPos is invalid or you give us only one.
  * We take pos by value because we sort it.
  * If two pos in the vector are equal, we return an empty Path there, unless 'remove_empty'*/
 std::vector<Path> SimpleContour::SplitAt(std::vector<PathPos> pos, bool remove_empty) const
@@ -1577,7 +1549,7 @@ Range SimpleContour::CutWithTangent(const Edge &e, std::pair<XY, XY> &from, std:
  *
  * We return all crosspoints of an infinite line specified by `A` and `B`.
  * We return them in `pos` terms, that is, at `A` we return 0, at `B` we return 1 and
- * linearly intrapolate and extrapolate between them and outside, respectively.
+ * linearly interpolate and extrapolate between them and outside, respectively.
  *
  * @param [in] A One point of the infinite line.
  * @param [in] B Another point of the infinite line.
@@ -1587,7 +1559,7 @@ void SimpleContour::Cut(const XY &A, const XY &B, DoubleMap<bool> &map) const
 {
     const Range ret = GetBoundingBox().Cut(A, B); //also tests for A==B or invalid bb, which catches empty "this"
     if (ret.IsInvalid()) return;
-    const Edge s(A+(B-A)*ret.from, A+(B-A)*(ret.till+0.1));
+    const Edge s(A+(B-A)*(ret.from-0.1), A+(B-A)*(ret.till+0.1));
     DoubleMap<bool> local;
     XY dummy1[2];
     double pos[2], dummy2[2];
@@ -1602,6 +1574,7 @@ void SimpleContour::Cut(const XY &A, const XY &B, DoubleMap<bool> &map) const
                 local.Set((p-A.y)/(B.y-A.y), true);
             }
     if (local.size()<=1) return;
+    const bool clockwise = GetClockWise();
     for (DoubleMap<bool>::const_iterator i=++local.begin(), j=local.begin(); i!=local.end(); j=i,i++)
         if (inside(IsWithin(A+(B-A)*(i->first+j->first)/2)))
             map.Set(Range(j->first, i->first), clockwise);
@@ -1619,8 +1592,9 @@ std::string SimpleContour::Dump(bool precise) const {
 
 SimpleContour SimpleContour::UnsafeMake(Path&& v) {
     SimpleContour ret;
+    ret.dirty_bb();
+    ret.area = infa;
     ret.edges.swap(v);
-    ret.clockwise_fresh = ret.boundingBox_fresh = ret.area_fresh = false;
     return ret;
 }
 
