@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -22,7 +22,6 @@
 
 #if !defined(STRINGPARSE_H)
 #define STRINGPARSE_H
-#include <set>
 #include "cairo.h"
 #include "cgen_attribute.h"
 #include "csh.h"
@@ -148,7 +147,7 @@ class StringFormat {
 public:
     /** Parses and described the content of a \ S(name,height,fillcolor) escape*/
     struct ShapeEscape {
-        bool Parse(std::string_view text, int startloc_csh, FileLineCol startloc_parse, const ShapeCollection *shapes, const ColorSet *colors, MscError *error, Csh *csh);
+        bool Parse(std::string_view text, CharByteIndex startloc_csh, FileLineCol startloc_parse, const ShapeCollection *shapes, const ColorSet *colors, MscError *error, Csh *csh);
         std::string Print(const ShapeCollection &shapes) const;
         CshPos loc_name;        ///<The location of the 'name' component, first_pos==-1, if none
         CshPos loc_height;      ///<The location of the 'height' component, first_pos==-1, if none
@@ -288,7 +287,7 @@ public:
      * @{ */
     static bool HasEscapes(std::string_view text);
     static bool HasLinkEscapes(std::string_view text);
-    static EEscapeHintType ExtractCSH(int startpos, std::string_view text, Csh &csh);
+    static EEscapeHintType ExtractCSH(CharByteIndex startpos, std::string_view text, Csh &csh);
     static void AddNumbering(string &label, std::string_view num, std::string_view pre_num_post);
     static void ExpandReferences(string &text, Chart *chart, FileLineCol linenum,
                                  const StringFormat *basic, bool references, 
