@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2023 Zoltan Turanyi
+	Copyright (C) 2008-2024 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -49,12 +49,12 @@ std::string FlowChart::GetLanguageDefaultText() const {
         "else goto turn;\n";
 }
 
-std::unique_ptr<Csh> FlowChart::CshFactory(Csh::FileListProc proc) const
+std::unique_ptr<Csh> FlowChart::CshFactory(Csh::FileListProc proc, const LanguageCollection* languages) const
 {
-    return std::make_unique<FlowCsh>(proc);
+    return std::make_unique<FlowCsh>(proc, languages);
 }
 
-FlowChart::FlowChart(FileReadProcedure *p, void *param) : ChartBase(p, param)
+FlowChart::FlowChart(FileReadProcedure *p, void *param, const LanguageCollection* languages) : ChartBase(p, param, languages)
 {
     Contexts.emplace_back(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol());  //creates new, plain context
     Designs.emplace("plain", Contexts.back());
@@ -116,7 +116,7 @@ bool FlowChart::ApplyForcedDesign(const string & name)
         return false;
     ignore_designs = true;
     Contexts.back().ApplyContextContent(i->second);
-    return false;
+    return Chart::ApplyForcedDesign(name);
 }
 
 /** Adds a chart option. Currently this is only 'pedantic'.
@@ -132,15 +132,13 @@ bool FlowChart::ApplyForcedDesign(const string & name)
     return false;
 }
 
-unsigned FlowChart::ParseText(std::string_view input, std::string_view filename)
+void FlowChart::ParseText(std::string_view input, const FileLineCol& first_char)
 {
-    unsigned ret = current_file = Error.AddFile(filename);
     //if you support progres::
     //Progress.RegisterParse(len);
-    sv_reader<false> reader(input, current_file);
+    sv_reader<false> reader(input, first_char);
     procedure_parse_helper proc_helper;
     flow_compile_parse(*this, proc_helper, reader); //return value ignored
-    return ret;
 }
 
 
@@ -481,10 +479,8 @@ void FlowChart::RegisterAllLabels()
 {
 }
 
-void FlowChart::DrawComplete(Canvas & c, bool pageBreaks, unsigned page)
+void FlowChart::Draw(Canvas & c, bool pageBreaks, unsigned page)
 {
-    if (page>pageBreakData.size() || pageBreakData.size()==0)
-        return;
     if (page) {
         //draw page X
     } else {
@@ -498,8 +494,6 @@ void FlowChart::DrawComplete(Canvas & c, bool pageBreaks, unsigned page)
             for (auto &n : sl.nodes)
                 n.Draw(c);
     }
-    //TODO: make sure you fill bkFill to help determining bkground for headers and footers.
-    DrawHeaderFooter(c, page);
 }
 
 void FlowChart::SetToEmpty()
