@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -31,13 +31,13 @@ namespace flow {
 class FlowCsh : public Csh
 {
 public:
-    explicit FlowCsh(Csh::FileListProc proc);
+    explicit FlowCsh(Csh::FileListProc proc, const LanguageCollection* languages);
     ~FlowCsh() override = default;
     std::unique_ptr<Csh> Clone() const override { return std::make_unique<FlowCsh>(*this); }
 
     // Inherited via Csh
     void FillNamesHints() override;
-    void ParseText(std::string&& input, int cursor_p, bool pedantic) override;
+    void ParseText(std::string&& input, CharByteIndex first_char, CharByteIndex cursor_p, bool pedantic) override;
 
 };
 
