@@ -127,8 +127,10 @@ void TextEditor::SetFonts(ImFont *regular, ImFont *bold, ImFont *italic, ImFont
 }
 
 
-//Each newline counts as one character
-//TABs are counted as one character in 'pos' but as mTabSize in the ret value
+/** Returns the corrdinates of the nth character in the file.
+ * Each newline counts as one character.
+ * TABs are counted as one character in 'pos' but as mTabSize in the ret value
+ * @param [in] char_pos The (zero-based index of the character whose coordinates we seek) */
 TextEditor::Coordinates TextEditor::GetCoordinate(int char_pos) noexcept {
 	if (char_pos < 0) return {0,0};
 	if (mLines.empty()) return {0,0};
@@ -144,7 +146,30 @@ TextEditor::Coordinates TextEditor::GetCoordinate(int char_pos) noexcept {
 	return {int(mLines.size() - 1), int(mLines.back().num_chars())};
 }
 
+/** Returns the zero-based index of the *character* before the cursor.
+ * If the cursor is at the beginning of the file, we return zero.
+ * If the cursor is beyond the end of the file, we return -1.*/
+std::pair<int, int> TextEditor::GetCharBytePos(const Coordinates& a_) const noexcept {
+    Coordinates a = SanitizeCoordinates(a_);
+    if (a.mLine >= (int)mLines.size()) return { -1, -1 };
+    if (a.mColumn > (int)mLines[a.mLine].num_chars()) return { -1, -1 };
+    std::pair<int, int> pos = { 0,0 };
+    for (int line = 0; line < a.mLine; line++) {
+        pos.first += mLines[line].num_chars() + 1; //+1 for newline
+        pos.second += mLines[line].size() + 1; //+1 for newline
+    }
+    pos.first += a.mColumn;
+    int i = 0, chars = a.mColumn;
+    for (; i < (int)mLines[a.mLine].size() && chars>0; chars--) 
+        i += UTF8TrailingBytes(mLines[a.mLine][i].mChar) + 1;
+    if (chars) return { -1,-1 };
+    pos.second += i;
+    return pos;
+}
 
+/** Returns the zero-based index of the *character* before the cursor.
+ * If the cursor is at the beginning of the file, we return zero.
+ * If the cursor is beyond the end of the file, we return -1.*/
 int TextEditor::GetCharPos(const Coordinates &a_) const noexcept {
 	Coordinates a = SanitizeCoordinates(a_);
 	if (a.mLine>=(int)mLines.size()) return -1;
@@ -475,7 +500,7 @@ TextEditor::Coordinates TextEditor::ScreenPosToCoordinates(const ImVec2& aPositi
 				buf[i] = '\0';
 				columnWidth = mFonts[0]->CalcTextSizeA(mFontSize, FLT_MAX, -1.0f, buf).x;
 			}
-			if (mTextStart + columnX + columnWidth * 0.5f > local.x)
+			if (mTextStart + columnX + columnWidth > local.x)
 				break;
 			columnX += columnWidth;
 			columnCoord++;
@@ -1252,13 +1277,14 @@ void TextEditor::EnterCharacter(ImWchar aChar, bool aShift)
 {
 	assert(!mReadOnly);
 
-	UndoRecord u;
+    const bool had_selection = HasSelection();
+    if (!had_selection)
+        mState.mSelectionStart = mState.mSelectionEnd = GetActualCursorCoordinates();
+    
+    UndoRecord u;
 	u.mBefore = mState;
 
 	if (aChar == '\t') {
-		const bool had_selection = HasSelection();
-		if (!had_selection)
-			mState.mSelectionStart = mState.mSelectionEnd = GetActualCursorCoordinates();
 		const int lStartLenBefore = mLines[mState.mSelectionStart.mLine].size();
 		const int lEndLenBefore = mLines[mState.mSelectionEnd.mLine].size();
 		u.mRemovedStart = {mState.mSelectionStart.mLine, 0};
@@ -1493,6 +1519,7 @@ void TextEditor::DeleteSelection()
 
 	SetSelection(mState.mSelectionStart, mState.mSelectionStart);
 	SetCursorPosition(mState.mSelectionStart);
+	mInteractiveStart = mInteractiveEnd = mState.mCursorPosition;
 	Colorize(mState.mSelectionStart.mLine, 1);
 }
 
@@ -1616,7 +1643,7 @@ void TextEditor::MoveLeft(int aAmount, bool aSelect, bool aWordMode)
 	}
 	else
 		mInteractiveStart = mInteractiveEnd = mState.mCursorPosition;
-	SetSelection(mInteractiveStart, mInteractiveEnd, aSelect && aWordMode ? SelectionMode::Word : SelectionMode::Normal);
+	SetSelection(mInteractiveStart, mInteractiveEnd, SelectionMode::Normal);
 
 	EnsureCursorVisible();
 }
@@ -1648,8 +1675,18 @@ void TextEditor::MoveRight(int aAmount, bool aSelect, bool aWordMode)
 		{
 			cindex += UTF8CharLength(line[cindex].mChar);
 			mState.mCursorPosition = Coordinates(lindex, GetCharacterColumn(lindex, cindex));
-			if (aWordMode)
-				mState.mCursorPosition = FindNextWord(mState.mCursorPosition);
+            if (aWordMode) {
+                mState.mCursorPosition = FindWordEnd(mState.mCursorPosition);
+                //Step through spaces
+                while (mState.mCursorPosition.mLine < (int)mLines.size()) 
+                    if ((int)mLines[mState.mCursorPosition.mLine].size() <= mState.mCursorPosition.mColumn) {
+                        mState.mCursorPosition.mColumn = 0;
+                        mState.mCursorPosition.mLine++;
+                    } else if (mLines[mState.mCursorPosition.mLine][mState.mCursorPosition.mColumn].mChar == ' ')
+                        mState.mCursorPosition.mColumn++;
+                    else
+                        break;
+            }
 		}
 	}
 
@@ -1710,24 +1747,21 @@ void TextEditor::MoveHome(bool aSelect)
 	auto oldPos = mState.mCursorPosition;
 	SetCursorPosition(Coordinates(mState.mCursorPosition.mLine, 0));
 
-	if (mState.mCursorPosition != oldPos)
+	if (aSelect)
 	{
-		if (aSelect)
-		{
-			if (oldPos == mInteractiveStart)
-				mInteractiveStart = mState.mCursorPosition;
-			else if (oldPos == mInteractiveEnd)
-				mInteractiveEnd = mState.mCursorPosition;
-			else
-			{
-				mInteractiveStart = mState.mCursorPosition;
-				mInteractiveEnd = oldPos;
-			}
-		}
+		if (oldPos == mInteractiveStart)
+			mInteractiveStart = mState.mCursorPosition;
+		else if (oldPos == mInteractiveEnd)
+			mInteractiveEnd = mState.mCursorPosition;
 		else
-			mInteractiveStart = mInteractiveEnd = mState.mCursorPosition;
-		SetSelection(mInteractiveStart, mInteractiveEnd);
+		{
+			mInteractiveStart = mState.mCursorPosition;
+			mInteractiveEnd = oldPos;
+		}
 	}
+	else
+		mInteractiveStart = mInteractiveEnd = mState.mCursorPosition;
+	SetSelection(mInteractiveStart, mInteractiveEnd);
 }
 
 void TextEditor::MoveEnd(bool aSelect)
@@ -1735,24 +1769,21 @@ void TextEditor::MoveEnd(bool aSelect)
 	auto oldPos = mState.mCursorPosition;
 	SetCursorPosition(Coordinates(mState.mCursorPosition.mLine, GetLineMaxColumn(oldPos.mLine)));
 
-	if (mState.mCursorPosition != oldPos)
+	if (aSelect)
 	{
-		if (aSelect)
-		{
-			if (oldPos == mInteractiveEnd)
-				mInteractiveEnd = mState.mCursorPosition;
-			else if (oldPos == mInteractiveStart)
-				mInteractiveStart = mState.mCursorPosition;
-			else
-			{
-				mInteractiveStart = oldPos;
-				mInteractiveEnd = mState.mCursorPosition;
-			}
-		}
+		if (oldPos == mInteractiveEnd)
+			mInteractiveEnd = mState.mCursorPosition;
+		else if (oldPos == mInteractiveStart)
+			mInteractiveStart = mState.mCursorPosition;
 		else
-			mInteractiveStart = mInteractiveEnd = mState.mCursorPosition;
-		SetSelection(mInteractiveStart, mInteractiveEnd);
+		{
+			mInteractiveStart = oldPos;
+			mInteractiveEnd = mState.mCursorPosition;
+		}
 	}
+	else
+		mInteractiveStart = mInteractiveEnd = mState.mCursorPosition;
+	SetSelection(mInteractiveStart, mInteractiveEnd);
 }
 
 void TextEditor::Delete(bool aWordMode)
@@ -1914,16 +1945,18 @@ void TextEditor::Copy()
 	if (HasSelection())
 	{
 		ImGui::SetClipboardText(GetSelectedText().c_str());
+        mFullLineOnClipboard = {};
 	}
 	else
 	{
 		if (!mLines.empty())
 		{
-			std::string str;
+            mFullLineOnClipboard = {};
 			auto& line = mLines[GetActualCursorCoordinates().mLine];
 			for (auto& g : line)
-				str.push_back(g.mChar);
-			ImGui::SetClipboardText(str.c_str());
+                mFullLineOnClipboard.push_back(g.mChar);
+            mFullLineOnClipboard.push_back('\n');
+			ImGui::SetClipboardText(mFullLineOnClipboard.c_str());
 		}
 	}
 }
@@ -1938,11 +1971,11 @@ void TextEditor::Cut()
 	{
 		if (HasSelection())
 		{
-			UndoRecord u;
-			u.mBefore = mState;
-			u.mRemoved = GetSelectedText();
-			u.mRemovedStart = mState.mSelectionStart;
-			u.mRemovedEnd = mState.mSelectionEnd;
+            UndoRecord u;
+            u.mBefore = mState;
+            u.mRemoved = mFullLineOnClipboard;
+            u.mRemovedStart = mState.mSelectionStart;
+            u.mRemovedEnd = mState.mSelectionEnd;
 
 			Copy();
 			DeleteSelection();
@@ -1950,12 +1983,34 @@ void TextEditor::Cut()
 			u.mAfter = mState;
 			AddUndo(u);
 		}
+        else
+        {
+            Copy();
+            UndoRecord u;
+            u.mBefore = mState;
+            u.mRemoved = mFullLineOnClipboard;
+            const int line = GetActualCursorCoordinates().mLine;
+            u.mRemovedStart = { line, 0 };
+            if (line + 1 < (int)mLines.size())
+                u.mRemovedEnd = { line + 1 , 0 };
+            else
+                u.mRemovedEnd = { line, GetLineMaxColumn(line)};
+
+            mLines.erase(mLines.begin() + line);
+
+            const Coordinates n = line < (int)mLines.size() ? Coordinates(line, 0) : Coordinates(line - 1, GetLineMaxColumn(line - 1));
+
+            u.mAfter = mState = { n, n, n };
+            AddUndo(u);
+        }
 	}
 }
 
 void TextEditor::Paste()
 {
-	Insert(ImGui::GetClipboardText());
+    if (!HasSelection() && mFullLineOnClipboard.size() && ImGui::GetClipboardText() == mFullLineOnClipboard)
+        mState.mCursorPosition.mColumn = mState.mSelectionStart.mColumn = mState.mSelectionEnd.mColumn = 0;
+    Insert(ImGui::GetClipboardText());
 }
 
 void TextEditor::Insert(const char *txt) {
