@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2023 Zoltan Turanyi
+	Copyright (C) 2008-2024 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -110,8 +110,8 @@ std::string ClusterCollapseMap::Serialize() const
     return s;
 }
 
-GraphChart::GraphChart(FileReadProcedure *p, void *param)
-    : ChartBase(p, param), last_subgraphattrs(EStyleType::ELEMENT, GraphStyle::GRAPH), cluser_counter(0)
+GraphChart::GraphChart(FileReadProcedure *p, void *param, const LanguageCollection* languages)
+    : ChartBase(p, param, languages), last_subgraphattrs(EStyleType::ELEMENT, GraphStyle::GRAPH), cluser_counter(0)
 {
     Contexts.emplace_back(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol());  //creates new, plain context
     Designs.emplace("plain", Contexts.back());
@@ -255,14 +255,15 @@ void GraphChart::AddCommandLineOption(const Attribute& /*a*/)
 {
 }
 
-void GraphChart::SetDefaultFont(const std::string &face, const std::string& /*lang*/,
-                                const FileLineColRange &l) {
+void GraphChart::SetDefaultFont(const std::string &face, const std::string& lang, const FileLineColRange& l) {
     if (face.length()) {
         for (GraphContext& context : Contexts)
             context.SetDefaultFont(face, this, l);
         for (auto& [name, design] : Designs)
             design.SetDefaultFont(face, this, l);
     }
+    //We skip calling ChartBase<>::SetDefaultFont()
+    Chart::SetDefaultFont(face, lang, l);
 }
 
 bool GraphChart::DeserializeGUIState(std::string_view s)
@@ -314,7 +315,7 @@ bool GraphChart::ApplyForcedDesign(const string & name)
         return false;
     ignore_designs = true;
     Contexts.back().ApplyContextContent(i->second);
-    return false;
+    return Chart::ApplyForcedDesign(name);
 }
 
 /** Apply a layout algorithm selected by the user on the ribbon or
@@ -399,17 +400,16 @@ void GraphChart::AddAttributeListToStyleList(const GraphStyle * s,
      delete al;
  }
 
-unsigned GraphChart::ParseText(std::string_view input, std::string_view filename)
+void GraphChart::ParseText(std::string_view input, const FileLineCol& first_char)
 {
+    current_file = first_char.file;
     if (do_orig_lang)
-        return orig_lang_ParseText(input, filename);
+        return orig_lang_ParseText(input);
     parse_stack.clear();
     last_subgraphname.clear();
-    unsigned ret = current_file = Error.AddFile(filename);
-    sv_reader<false> reader(input, current_file);
+    sv_reader<false> reader(input, first_char);
     gv_parser_helper_compile proc_helper;
     graph_compile_parse(*this, proc_helper, reader); //return value ignored
-    return ret;
 }
 
 /** Collapse the listed named clusters and fill in
@@ -571,7 +571,7 @@ void GraphChart::RegisterAllLabels()
 {
 }
 
-void GraphChart::DrawComplete(Canvas & c, bool pageBreaks, unsigned page)
+void GraphChart::Draw(Canvas & c, bool pageBreaks, unsigned page)
 {
     if (page>Graphs.size() || pageBreakData.size()==0 || Graphs.empty()) {
         c.Fill(total, {});
@@ -599,7 +599,6 @@ void GraphChart::DrawComplete(Canvas & c, bool pageBreaks, unsigned page)
             GG.Draw(c, Shapes, EGraphDrawPass::EDGE);
         }
     }
-    DrawHeaderFooter(c, page);
 }
 
 void GraphChart::SetToEmpty()
@@ -628,9 +627,8 @@ void GraphChart::orig_lang_clear()
 }
 
 
-unsigned GraphChart::orig_lang_ParseText(std::string_view input, std::string_view filename)
+void GraphChart::orig_lang_ParseText(std::string_view input)
 {
-    unsigned ret = current_file = Error.AddFile(filename);
     orig_lang_clear();
     //We use GG here just to store canvas and chart
     Graph GG;
@@ -646,7 +644,6 @@ unsigned GraphChart::orig_lang_ParseText(std::string_view input, std::string_vie
     GG._chart = nullptr;
     GG._err_msg_accum.clear();
     Graph::_current = nullptr;
-    return ret;
 }
 
 void GraphChart::orig_lang_CompleteParse(bool /*autoPaginate*/,
@@ -722,9 +719,9 @@ std::string GraphChart::GetLanguageDefaultText() const
         "}\n";
 }
 
-std::unique_ptr<Csh> GraphChart::CshFactory(Csh::FileListProc proc) const
+std::unique_ptr<Csh> GraphChart::CshFactory(Csh::FileListProc proc, const LanguageCollection* languages) const
 {
-    return std::make_unique<GraphCsh>(proc);
+    return std::make_unique<GraphCsh>(proc, languages);
 }
 
 std::map<std::string, std::string> GraphChart::RegisterLibraries() const
