@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -30,7 +30,7 @@
 #include "gvplugin_render.h"
 #include "gvplugin_device.h"
 #include "parser_tools.h" //for remove_head_tail_whitespace()
-#include "chartbase.h"
+#include "graphchart.h"
 #include "gvgraphs.h"
 
 #ifdef _WIN32
@@ -283,7 +283,7 @@ void GraphElement::PostPosProcess(Canvas &canvas, Chart * ch)
 
 /** The user-specified error function we give to DOT, to channel DOT errors into Chart::Error
  *  We collect the reported error text until we hit a '\n' (we treat the whole line as a single error)
- *  We try to figure out from the DOT error message, where the error occured inside the file.
+ *  We try to figure out from the DOT error message, where the error occurred inside the file.
  *   - For syntax errors, we use the term "near line xy".
  *   - For missing ports we use the term "node XY, port WZ unrecognized"
  *   - Else we look for attribute names and values in all objects to find if a bad attribute value is
@@ -481,18 +481,17 @@ void BindAttrs(Agraph_t *G, int type, const std::set<std::string> &attrs)
 /** Create the graph in graphviz and call the appropriate layout engine.
  * Collect errors in chart->Error. Does NOT fill in 'boundingBox'.
  * Call GetGv() to get the Agraph_t* to manipulate. nullptr on a problem.*/
-void Graph::Layout(Chart *chart)
+void Graph::Layout(GraphChart *chart)
 {
-    AddToGvGraph();
-    auto i = style.read().attributes.find("layout");
-    const char * layout = i==style.read().attributes.end() ? "dot" : i->second.value.c_str();
     _ASSERT(_current==nullptr);
     _canvas = nullptr;
     _chart = chart;
     _err_msg_accum.clear();
     _current = this;
-    std::stringbuf ss;
     agseterrf(Graph::DotError);
+    AddToGvGraph();
+    auto i = style.read().attributes.find("layout");
+    const char* layout = i == style.read().attributes.end() ? "dot" : i->second.value.c_str();
     int res = gvLayout(Gvc, in_gv, layout);
     agseterrf(nullptr);
     _canvas = nullptr;
@@ -510,7 +509,7 @@ void Graph::Layout(Chart *chart)
 /** Collect drawing instructions from graphviz.
  * Assumes a laid out graph. Errors go to chart->Error.
  * Fills in 'boundingBox'. Deletes the graph from cgraph.*/
-void Graph::PseudoRenderFrom(Chart*chart)
+void Graph::PseudoRenderFrom(GraphChart*chart)
 {
     boundingBox.MakeInvalid();
     if (in_gv) {
@@ -601,21 +600,22 @@ void Graph::AddToGvGraph()
 
     /* Add subgraphs, but delete those that are empty - or graphviz crashes */
     RemoveEmptySubGraphs();
-    for (auto &pSubGraph: subgraphs) {
+    for (auto &pSubGraph: subgraphs) 
         pSubGraph->AddToGvGraph();
-    }
 
     /* Add nodes not yet added */
     for (auto &pNode : nodes)
         if (pNode->GetGv()==nullptr) {
             pNode->AddToGvGraph(nullptr);
-            agbindrec(pNode->GetGv(), "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);	//node custom data
+            if (pNode->GetGv())
+                agbindrec(pNode->GetGv(), "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);	//node custom data
         }
 
     /* Connect nodes */
     for (auto &pEdge : edges) {
         pEdge->AddToGvGraph();
-        agbindrec(pEdge->GetGv(), "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);	//edge custom data
+        if (pEdge->GetGv())
+            agbindrec(pEdge->GetGv(), "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);	//edge custom data
     }
 }
 
@@ -859,6 +859,8 @@ void GraphNode::Draw(Canvas & canvas, const ShapeCollection &shapes, EGraphDrawP
 void GraphEdge::AddToGvGraph() const
 {
     in_gv = agedge(graph.GetGv(), n1->GetGv(), n2->GetGv(), name.length() ? name.c_str() : nullptr, 1);
+    if (!in_gv) 
+        return;
     Agsym_t *attr;
     if (port1.length()) {
         //bind TAILPORT_ID attribute
@@ -1160,12 +1162,18 @@ void cover_textspan(GVJ_t *job, pointf p, textspan_t * span)
     canvas_cover_textspan(false, job, p, span);
 }
 
+#if GRAPHVIZ_VER >= 1000
+void cover_polygon(GVJ_t* job, pointf* A, size_t n, int filled)
+#else
 void cover_polygon(GVJ_t * job, pointf * A, int n, int filled)
+#endif
 {
     canvas_cover_polygon(false, job, A, n, filled);
 }
 
-#if GRAPHVIZ_VER >= 800
+#if GRAPHVIZ_VER >= 1000
+void cover_beziercurve(GVJ_t* job, pointf* A, size_t n, int filled)
+#elif GRAPHVIZ_VER >= 800
 void cover_beziercurve(GVJ_t* job, pointf* A, int n, int filled)
 #else
 void cover_beziercurve(GVJ_t * job, pointf * A, int n, int /*arrow_at_start*/, int /*arrow_at_end*/, int filled)
@@ -1175,7 +1183,11 @@ void cover_beziercurve(GVJ_t * job, pointf * A, int n, int /*arrow_at_start*/, i
     canvas_cover_beziercurve(false, job, A, n, filled);
 }
 
+#if GRAPHVIZ_VER >= 1000
+void cover_polyline(GVJ_t* job, pointf* A, size_t n)
+#else
 void cover_polyline(GVJ_t * job, pointf * A, int n)
+#endif
 {
     canvas_cover_polyline(false, job, A, n);
 }
@@ -1212,9 +1224,9 @@ gvrender_engine_t cover_render_engine = {
     cover_textspan,     //(*textspan) (GVJ_t * job, pointf p, textspan_t * span);
     nullptr,               //(*resolve_color) (GVJ_t * job, gvcolor_t * color);
     cover_ellipse,      //(*ellipse) (GVJ_t * job, pointf * A, int filled);
-    cover_polygon,      //(*polygon) (GVJ_t * job, pointf * A, int n, int filled);
-    cover_beziercurve,  //(*beziercurve) (GVJ_t * job, pointf * A, int n, int arrow_at_start, int arrow_at_end, int); after graphviz 7.0: (GVJ_t * job, pointf * A, int n, int arrow_at_start, int arrow_at_end, int);
-    cover_polyline,     //(*polyline) (GVJ_t * job, pointf * A, int n);
+    cover_polygon,      //(*polygon) (GVJ_t * job, pointf * A, size_t n, int filled);
+    cover_beziercurve,  //(*beziercurve) (GVJ_t * job, pointf * A, size_t n, int arrow_at_start, int arrow_at_end, int); after graphviz 7.0: (GVJ_t * job, pointf * A, int n, int arrow_at_start, int arrow_at_end, int);
+    cover_polyline,     //(*polyline) (GVJ_t * job, pointf * A, size_t n);
     nullptr,               //(*comment) (GVJ_t * job, char *comment);
     nullptr,               //(*library_shape) (GVJ_t * job, char *name, pointf * A, int n, int filled);
 };
@@ -1278,12 +1290,18 @@ void canvas_textspan(GVJ_t *job, pointf p, textspan_t * span)
     canvas_cover_textspan(true, job, p, span);
 }
 
+#if GRAPHVIZ_VER >= 1000
+void canvas_polygon(GVJ_t* job, pointf* A, size_t n, int filled)
+#else
 void canvas_polygon(GVJ_t * job, pointf * A, int n, int filled)
+#endif
 {
     canvas_cover_polygon(true, job, A, n, filled);
 }
 
-#if GRAPHVIZ_VER >= 800
+#if GRAPHVIZ_VER >= 1000
+void canvas_beziercurve(GVJ_t* job, pointf* A, size_t n, int filled)
+#elif GRAPHVIZ_VER >= 800
 void canvas_beziercurve(GVJ_t* job, pointf* A, int n, int filled)
 #else
 void canvas_beziercurve(GVJ_t * job, pointf * A, int n, int /*arrow_at_start*/, int /*arrow_at_end*/, int filled)
@@ -1292,7 +1310,11 @@ void canvas_beziercurve(GVJ_t * job, pointf * A, int n, int /*arrow_at_start*/,
     canvas_cover_beziercurve(true, job, A, n, filled);
 }
 
+#if GRAPHVIZ_VER >= 1000
+void canvas_polyline(GVJ_t* job, pointf* A, size_t n)
+#else
 void canvas_polyline(GVJ_t * job, pointf * A, int n)
+#endif
 {
     canvas_cover_polyline(true, job, A, n);
 }
@@ -1330,9 +1352,9 @@ gvrender_engine_t canvas_render_engine = {
     canvas_textspan,    //(*textspan) (GVJ_t * job, pointf p, textspan_t * span);
     nullptr,               //(*resolve_color) (GVJ_t * job, gvcolor_t * color);
     canvas_ellipse,     //(*ellipse) (GVJ_t * job, pointf * A, int filled);
-    canvas_polygon,     //(*polygon) (GVJ_t * job, pointf * A, int n, int filled);
-    canvas_beziercurve, //(*beziercurve) (GVJ_t * job, pointf * A, int n, int arrow_at_start, int arrow_at_end, int);
-    canvas_polyline,    //(*polyline) (GVJ_t * job, pointf * A, int n);
+    canvas_polygon,     //(*polygon) (GVJ_t * job, pointf * A, size_t n, int filled);
+    canvas_beziercurve, //(*beziercurve) (GVJ_t * job, pointf * A, size_t n, int arrow_at_start, int arrow_at_end, int);
+    canvas_polyline,    //(*polyline) (GVJ_t * job, pointf * A, size_t n);
     nullptr,               //(*comment) (GVJ_t * job, char *comment);
     nullptr,               //(*library_shape) (GVJ_t * job, char *name, pointf * A, int n, int filled);
 };
