@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -34,6 +34,13 @@
 
 /** @name Types & externs copied from cgraph.h: forward declare cgraph types, so we do not have to include the whole cgraph library here
  ** @{ */
+#undef CGRAPH_API
+#ifdef GVDLL //define this for Windows build (cgraph.h also needs it)
+#define CGRAPH_API __declspec(dllimport)
+#else
+#define CGRAPH_API
+#endif
+
 typedef struct Agraph_s Agraph_t;	///<graph, subgraph (or hyperedge) (copied from cgraph.h of graphviz)
 typedef struct Agnode_s Agnode_t;	///<node (atom) (copied from cgraph.h of graphviz)
 typedef struct Agedge_s Agedge_t;	///<node pair (copied from cgraph.h of graphviz)
@@ -41,18 +48,18 @@ typedef struct Agsym_s Agsym_t; ///<symbol (copied from cgraph.h of graphviz)
 typedef struct GVC_s GVC_t; ///<global gvc context (copied from cgraph.h of graphviz)
 typedef struct obj_state_s obj_state_t; ///<generic object (node, edge, graph, etc.) (copied from cgraph.h of graphviz)
 #if GRAPHVIZ_VER < 249
-extern "C" void *agbindrec(void *obj, char *name, unsigned int size, int move_to_front);
-extern "C" int agxset(void *obj, Agsym_t * sym, char *value);
-extern "C" int agset(void *obj, char *name, char *value);
-extern "C" Agsym_t *agattr(Agraph_t * g, int kind, char *name, char *value);
+extern "C" CGRAPH_API void *agbindrec(void *obj, char *name, unsigned int size, int move_to_front);
+extern "C" CGRAPH_API int agxset(void *obj, Agsym_t * sym, char *value);
+extern "C" CGRAPH_API int agset(void *obj, char *name, char *value);
+extern "C" CGRAPH_API Agsym_t *agattr(Agraph_t * g, int kind, char *name, char *value);
 #else
-extern "C" int agset(void *obj, char *name, const char *value);
-extern "C" Agsym_t *agattr(Agraph_t * g, int kind, char *name, const char *value);
+extern "C" CGRAPH_API int agset(void *obj, char *name, const char *value);
+extern "C" CGRAPH_API Agsym_t *agattr(Agraph_t * g, int kind, char *name, const char *value);
 #endif
-extern "C" char *agget(void *obj, char *name);
-extern "C" Agraph_t *agsubg(Agraph_t * g, char *name, int cflag);
-extern "C" Agnode_t *agnode(Agraph_t * g, char *name, int createflag);
-extern "C" Agedge_t *agedge(Agraph_t * g, Agnode_t * t, Agnode_t * h, char *name, int createflag);
+extern "C" CGRAPH_API char *agget(void *obj, char *name);
+extern "C" CGRAPH_API Agraph_t *agsubg(Agraph_t * g, char *name, int cflag);
+extern "C" CGRAPH_API Agnode_t *agnode(Agraph_t * g, char *name, int createflag);
+extern "C" CGRAPH_API Agedge_t *agedge(Agraph_t * g, Agnode_t * t, Agnode_t * h, char *name, int createflag);
 /** @} */
 
 /** @name wrappers for passing const char* parameters to cgraph functions
@@ -205,7 +212,7 @@ public:
     static Graph* _current;      ///<A static variable pointing to the Graph currently under parsing, layout or pseudo-rendering. Error messages and rendered drawing primitives are inserted into this.
     Canvas *_canvas = nullptr;   ///<The canvas during pseudo-rendering.
     std::string _err_msg_accum;  ///<We collect multi-segment DOT error messages in this.
-    Chart *_chart = nullptr;     ///<The chart to add errors to during layout or parsing.
+    GraphChart *_chart = nullptr;///<The chart to add errors to during layout or parsing.
     static int DotError(char *msg);
     /** @} */
 
@@ -281,8 +288,8 @@ public:
     bool RemoveEmptySubGraphs();
 
     void ShiftBy(const XY &) override;
-    void Layout(Chart*chart);
-    void PseudoRenderFrom(Chart*chart);
+    void Layout(GraphChart*chart);
+    void PseudoRenderFrom(GraphChart*chart);
     void PostPosProcess(Canvas &, Chart *ch) override;
     void AdjustBkArea(const Range& total_x) noexcept;
     void GraphCollectIsMapElements(Canvas& canvas, Chart& chart) const override;
@@ -536,4 +543,6 @@ GraphEdge *Graph::CreateEdge(GraphNode *_1, GraphNode *_2,
 
 } //namespace graph
 
+#undef CGRAPH_API
+
 #endif
