@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -296,8 +296,14 @@ public:
     bool IsStickToPrev() const { return stick_to_prev; }
     /** True if a note can be attached to this arc. */
     virtual bool CanBeNoted() const {return false;}
-    /** True if the entity is likely to produce height and can be the target of a vertical*/
+    /** True if the entity is likely to produce height and can be the target of a vertical or a byref symbol. */
     virtual bool CanBeAlignedTo() const { return false; }
+    /** True if this entity aligns to another entity by referring to it. 
+     * Such entities are laid out independently. */
+    virtual bool IsByRef() const { return false; }
+    /** For by ref symbols and verticals, it returns the element to
+     * align to. For others it is a nullptr.*/
+    virtual ArcBase* GetRefTarget() const { return nullptr; }
     /** True if implicitly generated entities (MscChart::AutoGenEntities) shall be placed *after* this element (like titles, etc) */
     virtual bool BeforeAutoGenEntities() const { return false; }
     /** If there is an indicator both before and after this element, we
@@ -305,7 +311,7 @@ public:
     virtual bool SeparatesIndicators() const { return true; }
     /** If this element is followed by a Note, it can store the note for later re-insertion.
      * This feature is useful for entity commands and arrow joins: when a series of
-     * instrcutions are merged and we may want to note only one of them.
+     * instructions are merged and we may want to note only one of them.
      * @returns true if we have stored the Note internally (and took ownership)*/
     virtual bool StoreNoteTemporarily(Note*) { return false; }
     /** After PostParseProcess, we can re-insert the notes stored above.*/
@@ -388,9 +394,17 @@ public:
      * when we know that the previous arrow do not disappear and a curly join is still needed.
      * Called only for elements that return can_curly_join==true in GetConnection()*/
     virtual void SetCurlyJoin(bool is_src [[maybe_unused]], ArcBase*, bool src_of_prev [[maybe_unused]]) { _ASSERT(0); } //This entity cannot participate in join series - should not have been called
+    /** Which side of the element do we query the centerline? */
+    enum class CenterlineSelector
+    {
+        AT_SRC,     ///<At the source of the arrow
+        AT_DST,     ///<At the destination of the arrow
+        LEFT_SIDE,  ///<At the phyiscal left side (can be source or dest, if those are meaningful)
+        RIGHT_SIDE  ///<At the phyiscal right side (can be source or dest, if those are meaningful)
+    };
     /** Get the y position of the end, for the purposes of joining to it.
     * Shall be called only after Layout()*/
-    virtual double GetCenterline(bool at_src [[maybe_unused]]) const { _ASSERT(0); return 0; } //This entity cannot participate in join series - should not have been called
+    virtual double GetCenterline(CenterlineSelector where [[maybe_unused]]) const { _ASSERT(0); return 0; } //This entity cannot participate in join series - should not have been called
     /** Get the activation status of the entity at the dst end of the arrow,
      * for the purposes of joining to it. Shall be called only after Layout(). */
     virtual double GetActSize(bool at_src [[maybe_unused]]) const { _ASSERT(0); return 0; } //This entity cannot participate in join series - should not have been called
@@ -407,7 +421,7 @@ public:
                                       Numbering &number, MscElement **note_target, ArcBase *vertical_target);
     /** Substitute name references in labels & process all escapes
      * See documentation for libmscgen for more info.*/
-    virtual void FinalizeLabels(Canvas &canvas);
+    virtual void FinalizeLabels(Canvas &) {}
     /** Collects vertical distances for hscale=auto mechanism.
      * See documentation for libmscgen for more info.*/
     virtual void Width(Canvas &/*canvas*/, DistanceRequirements &/*vdist*/) {}
@@ -415,10 +429,16 @@ public:
      * Calculates the height, and sets up the area at yPos==0,
      * add its cover to use at placement later to `cover`.
      * Cover or area does not include any spacing left around such as chart->emphVGapAbove.
+     * This function is not called for by reference verticals, symbol, nor for
+     * notes and comments.
      * See documentation for libmscgen for more info.*/
-    virtual void Layout(Canvas &canvas, AreaList *cover);
+    virtual void Layout(Canvas& canvas, AreaList* cover);
+    /** Calculate entity layout in relation to the coverage of the referred entity. 
+     * This function is called only for by reference verticals and symbols.*/
+    virtual bool LayoutByRef(Canvas& /*canvas*/, const AreaList& /*target_cover*/, AreaList* /*cover*/) { _ASSERT(0); return false; }
     /** After being placed below the element above us, here we can make final adjustments to layout.
      * This will be called before placing the next element below us.
+     * 'cover' contains any symbols drawn in reference to us.
      * Our yPos member is still not at its final coordinate.
      * (This is used for curly joins to add the 'curl'.)*/
     virtual void FinalizeLayout(Canvas &/*canvas*/, AreaList * /*cover*/) {}
@@ -487,7 +507,7 @@ public:
     ArrowEnding GetSidePos(bool is_src) const override;
     bool SetEndingDueToJoin(bool /*is_src*/, const ArrowEnding&, bool /*test_only*/)  override { _ASSERT(0); return false; } //Never unspecified
     void SetCurlyJoin(bool /*is_src*/, ArcBase*, bool /*src_of_prev*/) override { _ASSERT(0); } //never curly
-    double GetCenterline(bool /*at_src*/) const override { return GetIndicatorCover(XY(GetXCoord(), yPos)).y.MidPoint(); }
+    double GetCenterline(CenterlineSelector /*where*/) const override { return GetIndicatorCover(XY(GetXCoord(), yPos)).y.MidPoint(); }
     double GetActSize(bool at_src) const override;
 
     void Width(Canvas &canvas, DistanceRequirements &vdist) override;
@@ -580,7 +600,6 @@ struct VertXPos
     /** Options for horizontal position in relation to one or two entities */
     enum EPosType
     {
-        POS_INVALID = 0,///<Invalid value
         POS_AT,         ///<At the entity
         POS_CENTER,     ///<Halfway between two entities
         POS_THIRD_LEFT, ///<one third towards the neighbour on the left of an entity
@@ -594,8 +613,8 @@ struct VertXPos
     static bool IsOneEntityPos(EPosType t) { return t==POS_AT || t==POS_LEFT_BY || t==POS_RIGHT_BY || t==POS_LEFT_SIDE || t==POS_RIGHT_SIDE || t==POS_THIRD_LEFT || t==POS_THIRD_RIGHT;  }
     static bool IsTwoEntityPos(EPosType t) { return t==POS_CENTER; }
 
-    bool valid = true;           ///<True if the position is valid
-    EPosType pos = POS_INVALID;  ///<The type of the position
+    bool valid = false;          ///<True if the position is valid
+    EPosType pos;                ///<The type of the position
     EntityRef entity1 = nullptr; ///<The first (or only one) entity
     EntityRef entity2 = nullptr; ///<The second entity, used only for POS_CENTER
     FileLineColRange e1line;     ///<Location of the mention of the first entity in the input line (inside this vertpos fragment)
@@ -620,6 +639,14 @@ struct VertXPos
     bool IsTwoEntityPos() const { return IsTwoEntityPos(pos); }
 };
 
+/** By reference verticals has this as position. */
+struct VertRefPos
+{
+    ESide side = ESide::INVALID;  ///<RIGHT for 'rightof', LEFT for 'leftof'
+    std::string refname = {};     ///<The name of the referenced element. Empty if the previous one
+    FileLineCol refname_loc = {}; ///<The location of the refname (if any)
+};
+
 /** The base class for all arrows: Self, Dir, Block and Vertical*/
 class Arrow : public LabelledArc
 {
@@ -717,7 +744,7 @@ public:
     ArrowEnding GetSidePos(bool /*is_src*/) const override { return ArrowEnding(src, 0); }  //Before PostParseProcess() this is an iterator to AllEntities, after to ActiveEntities - OK
     bool SetEndingDueToJoin(bool /*is_src*/, const ArrowEnding&, bool /*test_only*/) override { _ASSERT(0); return false; } //Never unspecified
     void SetCurlyJoin(bool /*is_src*/, ArcBase*, bool /*src_of_prev*/) override { _ASSERT(0); } //never curly
-    double GetCenterline(bool at_src) const override { return yPos + (at_src ? src_height : src_height+wh.y); }
+    double GetCenterline(CenterlineSelector where) const override { return yPos + (where==CenterlineSelector::AT_SRC ? src_height : where==CenterlineSelector::AT_DST ? src_height+wh.y : src_height+wh.y/2); }
     double GetActSize(bool /*at_src*/) const override { return src_act; }
 
     void Width(Canvas &canvas, DistanceRequirements &vdist) override;
@@ -803,7 +830,7 @@ public:
     ArrowEnding GetSidePos(bool is_src) const override { return is_src ? src : dst; } //Before PostParseProcess() this is an iterator to AllEntities, after to ActiveEntities - OK
     bool SetEndingDueToJoin(bool is_src, const ArrowEnding &ending, bool test_only) override;
     void SetCurlyJoin(bool is_src, ArcBase *prev, bool src_of_prev) override;
-    double GetCenterline(bool is_src) const override { return yPos+GetCenterlineAtXPos(is_src ? sx : dx); }
+    double GetCenterline(CenterlineSelector where) const override { return yPos+GetCenterlineAtXPos(where==CenterlineSelector::AT_SRC ? sx : where == CenterlineSelector::AT_DST ? dx : (where == CenterlineSelector::LEFT_SIDE)==is_forward ? sx : dx); }
     double GetActSize(bool at_src) const override;
 
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
@@ -906,8 +933,9 @@ protected:
     EVerticalShape shape;    ///<The shape of the vertical
     VertXPos pos;            ///<The horizontal position, specified by the user
     WidthAttr width_attr;    ///<The value of the width attribute
-    const ArcBase *prev_arc; ///<The arc before us. Used only if no markers specified.
-    mutable bool forward;    ///<if the arrow is up to down.
+    VertRefPos by_ref;       ///<The position specified by the user, if we are a by ref vertical (no markers).
+    ArcBase *ref_target;     ///<The arc before us. Used only if no markers specified.
+    mutable bool forward;    ///<if the arrow is up to down. For byref verticals we set it already in the ctor.
     mutable bool left;       ///<True if we are at the left side of the entity (or we draw such)
     mutable double width;    ///<width of us (just the body for arrows)
     mutable double ext_width;///<Extra width of the arrowhead
@@ -922,9 +950,13 @@ protected:
     mutable Block clip_block;///<The clip of the line for brackets and pointers, ignoring arrowheads (for PPT)
     mutable Contour clip_line;  ///<The clip of the line for brackets and pointers
     mutable std::vector<Contour> outer_contours; ///<Calculated contour (only one element)
+    void DoLayout(Canvas& canvas, const AreaList* target_cover=nullptr);
 public:
-    /** Constructor for arrow or box symbols with two marker names (one can be nullptr)*/
+    bool layout_warning = true;///<If set, we emit layout warnings if the vertical is attached to a ParallelBlock with one_by_one_merge and spans _above_ the ParallelBlock 
+    /** Constructor for arrow or box symbols with two marker names */
     Vertical(const ArcTypePlusDir *t, std::string_view s, std::string_view d, MscChart *msc);
+    /** Constructor for a by reference vertical */
+    Vertical(const ArcTypePlusDir* t, gsl::owner<VertRefPos*> refpos, MscChart* msc);
     Arrow *AddSegment(ArrowSegmentData data, std::string_view m, const FileLineColRange &ml,
                       const FileLineColRange &l) override;
     Arrow *AddLostPos(gsl::owner<VertXPos*> pos, const FileLineColRange &l) override;
@@ -932,9 +964,11 @@ public:
     Arrow *AddEndPos(ArrowEnding *ending = nullptr) override;
     /** Add the parsed horizontal position (starting with the AT keyword) */
     Vertical* AddXpos(VertXPos *p);
+    /** Add the parsed reference (starting with the RIGHTOF or LEFTOF keywords) */
+    Vertical* AddRefPos(VertRefPos* p);
     /** Sets the shape of the vertical. Used only in parsing.*/
     void SetVerticalShape(EVerticalShape sh);
-    bool CanBeNoted() const override { return false; }
+    bool CanBeNoted() const override { return true; }
     bool CanBeAlignedTo() const override { return false; }
     bool StoreNoteTemporarily(Note*) override { return false; }
     void SetStyleBeforeAttributes(AttributeList *l) override;
@@ -942,17 +976,20 @@ public:
     bool AddAttribute(const Attribute &) override;
     static void AttributeNames(Csh &csh);
     static bool AttributeValues(std::string_view attr, Csh &csh, EVerticalShape shape);
+    bool IsByRef() const override;
+    ArcBase* GetRefTarget() const override { return IsByRef() ? ref_target : nullptr; }
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
                               Numbering &number, MscElement **note_target, ArcBase *vertical_target) override;
     void Width(Canvas &canvas, DistanceRequirements &vdist) override;
     void Layout(Canvas &canvas, AreaList *cover) override;
+    bool LayoutByRef(Canvas& canvas, const AreaList& target_cover, AreaList* cover) override;
     Range GetVisualYExtent(bool include_comments) const override;
     void ShiftBy(double y) override;
     Range GetFormalYExtent() const override { return Range(false); }
     double SplitByPageBreak(Canvas &/*canvas*/, double /*netPrevPageSize*/,
                             double /*pageBreak*/, bool &/*addCommandNewpage*/,
                             bool /*addHeading*/, ArcList &/*res*/) override {return -2;}
-    void PlaceWithMarkers(Canvas &canvas) override;
+    void PlaceWithMarkers(Canvas& canvas) override { if (!GetRefTarget()) DoLayout(canvas); }
     virtual void PostPosProcess(Canvas &, Chart *ch) override;
     void RegisterLabels() override;
     void CollectIsMapElements(Canvas &canvas) override;
