@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -151,7 +151,7 @@ public:
     ArrowEnding GetSidePos(bool is_src) const override;
     bool SetEndingDueToJoin(bool /*is_src*/, const ArrowEnding&, bool /*test_only*/)  override { _ASSERT(0); return false; } //Never unspecified
     void SetCurlyJoin(bool /*is_src*/, ArcBase*, bool /*src_of_prev*/) override { _ASSERT(0); } //never curly
-    double GetCenterline(bool /*at_src*/) const override { return yPos+total_height/2; }
+    double GetCenterline(CenterlineSelector /*where*/) const override { return yPos + total_height / 2; }
     double GetActSize(bool /*at_src*/) const override { _ASSERT(0); return 0; } //This never does curly joins
 
     ArcBase* PostParseProcess(Canvas &canvas, bool hide, EntityRef &left, EntityRef &right,
