@@ -72,6 +72,9 @@ lexing::classifier<char, token_state_symbol<TOKEN>> build_classifier(const lexin
     auto WS = just(' ') | just('\t');
     cl.insert(WS, TOKEN::TOK_WHITESPACE);
 
+    cl.insert(word(INLINE_OPEN_STR), TOKEN::TOK_OINLINE);
+    cl.insert(word(INLINE_CLOSE_STR), TOKEN::TOK_CINLINE);
+
     cl.insert(word("..."), {TOKEN::TOK_SPECIAL_ARC, EArcSymbol::DIV_DISCO});
     cl.insert(word("---"), {TOKEN::TOK_SPECIAL_ARC, EArcSymbol::DIV_DIVIDER});
     cl.insert(word("|||"), {TOKEN::TOK_SPECIAL_ARC, EArcSymbol::DIV_VSPACE});
@@ -205,8 +208,8 @@ lexing::classifier<char, token_state_symbol<TOKEN>> build_classifier(const lexin
     auto digit = range('0', '9');
     auto dot = just('.');
     auto sign = oneof("+-");
-    auto num = digit.plus() * (dot * digit.star()).optional(); //no sign
-    cl.insert(sign.optional() * num, TOKEN::TOK_NUMBER);
+    auto num = sign.optional() * digit.plus() * (dot * digit.star()).optional(); //no sign
+    cl.insert(num, TOKEN::TOK_NUMBER);
 
     /* Strings (UTF-8 allowed)
      * Starts with letter or underscore, may also include numbers or dots
@@ -242,8 +245,8 @@ lexing::classifier<char, token_state_symbol<TOKEN>> build_classifier(const lexin
     cl.insert(str * sign * num * (comma * num).optional(), TOKEN::TOK_COLORDEF);
      /* string,number[+-number].*/
     cl.insert(str * comma * num * (sign * num).optional(), TOKEN::TOK_COLORDEF);
-     /* number,number,number[,number] */
-    cl.insert(num * comma * num * comma * num * (comma * num).optional(), TOKEN::TOK_COLORDEF);
+     /* number,number[,number[,number]] */
+    cl.insert(num * comma * num * (comma * num).optional() * (comma * num).optional(), TOKEN::TOK_COLORDEF);
 
    /* Parameter names: $ followed by a string not ending with a dot.
     * A single '$" also matches.*/
@@ -317,11 +320,16 @@ lexing::classifier<char, token_state_symbol<TOKEN>> build_classifier(const lexin
     cl.insert(iword("if"), TOKEN::TOK_IF);
     cl.insert(iword("then"), TOKEN::TOK_THEN);
     cl.insert(iword("else"), TOKEN::TOK_ELSE);
+    
+    cl.insert(iword("around"), TOKEN::TOK_SYMBOLPOS_AROUND);
+    cl.insert(iword("leftof"), TOKEN::TOK_SYMBOLPOS_LEFTOF);
+    cl.insert(iword("rightof"), TOKEN::TOK_SYMBOLPOS_RIGHTOF);
+    cl.insert(iword("above"), TOKEN::TOK_SYMBOLPOS_ABOVE);
+    cl.insert(iword("below"), TOKEN::TOK_SYMBOLPOS_BELOW);
 
     cl = make_deterministic(cl);
     //cl = minimize(cl); //TODO: uncomment, when maphoon works.
     return cl;
-
 }
 
 //When we do Color Syntax Highlight parsing the chart object is Csh instead.
@@ -370,6 +378,43 @@ token_state_symbol<TOKEN> get_token(sv_reader<CSH>& inp, ChartOrCsh<CSH>& C, REA
             //comments does not change the state, keep using current one
             continue;
         }
+        case TOKEN::TOK_OINLINE: {
+            const auto loc_oinline = inp.template commit<false, false>(len);
+            loc = inp.template commit<false, false>(0);
+            const char *start_text_ptr = inp.view(0).data(), *end_text_ptr = start_text_ptr;
+            unsigned nesting_level = 0;
+            while (inp.has(1)) {
+                auto [int_token_state_symbol, len] = readandclassify(0, inp);
+                switch (TOKEN(int_token_state_symbol.token)) {
+                case TOKEN::TOK_OINLINE:
+                    nesting_level++;
+                    FALLTHROUGH;
+                default:
+                    if (!len) len = 1;
+                    end_text_ptr = inp.view(len).data() + len;
+                    loc.merge(inp.template commit<true, false>(len));
+                    break;
+                case TOKEN::TOK_CINLINE:
+                    if (nesting_level) {
+                        nesting_level--;
+                        end_text_ptr = inp.view(len).data() + len;
+                        loc.merge(inp.template commit<true, false>(len));
+                        break;
+                    }
+                    inp.template commit<true, false>(len);
+                    val.str = std::string_view(start_text_ptr, end_text_ptr - start_text_ptr);
+                    return TOKEN::TOK_INLINE;
+                }
+            }
+            //Unpaired OINLINE
+            if constexpr (CSH) {
+                C.AddCSH_Error(loc_oinline, "Unpaired beginning of inlined chart '" INLINE_OPEN_STR "'.");
+                C.AddCSH(loc, COLOR_COMMENT); //we have consumed the whole file, color as comment
+            } else
+                C.Error.Error(loc_oinline.start(), "Unpaired beginning of inlined chart '" INLINE_OPEN_STR "'.");
+            loc = loc_oinline;
+            return TOKEN::TOK_UNMATCHED_INLINE;
+        }
         case TOKEN::TOK_SPECIAL_ARC:
         case TOKEN::TOK_REL_TO:
         case TOKEN::TOK_REL_FROM:
@@ -394,6 +439,7 @@ token_state_symbol<TOKEN> get_token(sv_reader<CSH>& inp, ChartOrCsh<CSH>& C, REA
         case TOKEN::TOK_CPARENTHESIS:
         case TOKEN::TOK_ASTERISK:
         case TOKEN::TOK_PIPE_SYMBOL:
+        case TOKEN::TOK_CINLINE:
         case TOKEN::TOK_UNRECOGNIZED_CHAR:
         case TOKEN::TOK_EOF:
             loc = inp.template commit<false, false>(len);
@@ -474,6 +520,11 @@ token_state_symbol<TOKEN> get_token(sv_reader<CSH>& inp, ChartOrCsh<CSH>& C, REA
         case TOKEN::TOK_IF:
         case TOKEN::TOK_THEN:
         case TOKEN::TOK_ELSE:
+        case TOKEN::TOK_SYMBOLPOS_AROUND:
+        case TOKEN::TOK_SYMBOLPOS_ABOVE:
+        case TOKEN::TOK_SYMBOLPOS_BELOW:
+        case TOKEN::TOK_SYMBOLPOS_LEFTOF:
+        case TOKEN::TOK_SYMBOLPOS_RIGHTOF:
 
         //special tokens, whose value is their text
         case TOKEN::TOK_COLORDEF:
@@ -540,7 +591,7 @@ token_state_symbol<TOKEN> get_token(sv_reader<CSH>& inp, ChartOrCsh<CSH>& C, REA
             //scan "box" as TOK_COMMAND_BOX and not as TOK_STRING
             if (text.ends_with("..")) {
                 text.remove_suffix(2);
-                sv_reader<CSH> input(text, 0);
+                sv_reader<CSH> input(text, {});
                 std::tie(int_token_state_symbol, len) = readandclassify(0, input);
                 goto re_scanned;
             }
