@@ -124,61 +124,70 @@ extern int msc_csh_debug;
     TOK_SPECIAL_ARC = 290,         /* TOK_SPECIAL_ARC  */
     TOK_EMPH = 291,                /* TOK_EMPH  */
     TOK_EMPH_PLUS_PLUS = 292,      /* TOK_EMPH_PLUS_PLUS  */
-    TOK_COMMAND_HEADING = 293,     /* TOK_COMMAND_HEADING  */
-    TOK_COMMAND_NUDGE = 294,       /* TOK_COMMAND_NUDGE  */
-    TOK_COMMAND_NEWPAGE = 295,     /* TOK_COMMAND_NEWPAGE  */
-    TOK_COMMAND_DEFSHAPE = 296,    /* TOK_COMMAND_DEFSHAPE  */
-    TOK_COMMAND_DEFCOLOR = 297,    /* TOK_COMMAND_DEFCOLOR  */
-    TOK_COMMAND_DEFSTYLE = 298,    /* TOK_COMMAND_DEFSTYLE  */
-    TOK_COMMAND_DEFDESIGN = 299,   /* TOK_COMMAND_DEFDESIGN  */
-    TOK_COMMAND_DEFPROC = 300,     /* TOK_COMMAND_DEFPROC  */
-    TOK_COMMAND_REPLAY = 301,      /* TOK_COMMAND_REPLAY  */
-    TOK_COMMAND_SET = 302,         /* TOK_COMMAND_SET  */
-    TOK_COMMAND_INCLUDE = 303,     /* TOK_COMMAND_INCLUDE  */
-    TOK_COMMAND_BIG = 304,         /* TOK_COMMAND_BIG  */
-    TOK_COMMAND_BOX = 305,         /* TOK_COMMAND_BOX  */
-    TOK_COMMAND_PIPE = 306,        /* TOK_COMMAND_PIPE  */
-    TOK_COMMAND_MARK = 307,        /* TOK_COMMAND_MARK  */
-    TOK_COMMAND_MARK_SRC = 308,    /* TOK_COMMAND_MARK_SRC  */
-    TOK_COMMAND_MARK_DST = 309,    /* TOK_COMMAND_MARK_DST  */
-    TOK_COMMAND_PARALLEL = 310,    /* TOK_COMMAND_PARALLEL  */
-    TOK_COMMAND_OVERLAP = 311,     /* TOK_COMMAND_OVERLAP  */
-    TOK_COMMAND_INLINE = 312,      /* TOK_COMMAND_INLINE  */
-    TOK_VERTICAL = 313,            /* TOK_VERTICAL  */
-    TOK_VERTICAL_SHAPE = 314,      /* TOK_VERTICAL_SHAPE  */
-    TOK_AT = 315,                  /* TOK_AT  */
-    TOK_LOST = 316,                /* TOK_LOST  */
-    TOK_AT_POS = 317,              /* TOK_AT_POS  */
-    TOK_SHOW = 318,                /* TOK_SHOW  */
-    TOK_HIDE = 319,                /* TOK_HIDE  */
-    TOK_ACTIVATE = 320,            /* TOK_ACTIVATE  */
-    TOK_DEACTIVATE = 321,          /* TOK_DEACTIVATE  */
-    TOK_BYE = 322,                 /* TOK_BYE  */
-    TOK_START = 323,               /* TOK_START  */
-    TOK_BEFORE = 324,              /* TOK_BEFORE  */
-    TOK_END = 325,                 /* TOK_END  */
-    TOK_AFTER = 326,               /* TOK_AFTER  */
-    TOK_COMMAND_VSPACE = 327,      /* TOK_COMMAND_VSPACE  */
-    TOK_COMMAND_HSPACE = 328,      /* TOK_COMMAND_HSPACE  */
-    TOK_COMMAND_SYMBOL = 329,      /* TOK_COMMAND_SYMBOL  */
-    TOK_COMMAND_NOTE = 330,        /* TOK_COMMAND_NOTE  */
-    TOK_COMMAND_COMMENT = 331,     /* TOK_COMMAND_COMMENT  */
-    TOK_COMMAND_ENDNOTE = 332,     /* TOK_COMMAND_ENDNOTE  */
-    TOK_COMMAND_FOOTNOTE = 333,    /* TOK_COMMAND_FOOTNOTE  */
-    TOK_COMMAND_TITLE = 334,       /* TOK_COMMAND_TITLE  */
-    TOK_COMMAND_SUBTITLE = 335,    /* TOK_COMMAND_SUBTITLE  */
-    TOK_COMMAND_TEXT = 336,        /* TOK_COMMAND_TEXT  */
-    TOK_SHAPE_COMMAND = 337,       /* TOK_SHAPE_COMMAND  */
-    TOK_JOIN = 338,                /* TOK_JOIN  */
-    TOK_IF = 339,                  /* TOK_IF  */
-    TOK_THEN = 340,                /* TOK_THEN  */
-    TOK_ELSE = 341,                /* TOK_ELSE  */
-    TOK_MSCGEN_RBOX = 342,         /* TOK_MSCGEN_RBOX  */
-    TOK_MSCGEN_ABOX = 343,         /* TOK_MSCGEN_ABOX  */
-    TOK_UNRECOGNIZED_CHAR = 344,   /* TOK_UNRECOGNIZED_CHAR  */
-    TOK_WHITESPACE = 345,          /* TOK_WHITESPACE  */
-    TOK_NEWLINE = 346,             /* TOK_NEWLINE  */
-    TOK_COMMENT = 347              /* TOK_COMMENT  */
+    TOK_OINLINE = 293,             /* TOK_OINLINE  */
+    TOK_CINLINE = 294,             /* TOK_CINLINE  */
+    TOK_INLINE = 295,              /* TOK_INLINE  */
+    TOK_UNMATCHED_INLINE = 296,    /* TOK_UNMATCHED_INLINE  */
+    TOK_COMMAND_HEADING = 297,     /* TOK_COMMAND_HEADING  */
+    TOK_COMMAND_NUDGE = 298,       /* TOK_COMMAND_NUDGE  */
+    TOK_COMMAND_NEWPAGE = 299,     /* TOK_COMMAND_NEWPAGE  */
+    TOK_COMMAND_DEFSHAPE = 300,    /* TOK_COMMAND_DEFSHAPE  */
+    TOK_COMMAND_DEFCOLOR = 301,    /* TOK_COMMAND_DEFCOLOR  */
+    TOK_COMMAND_DEFSTYLE = 302,    /* TOK_COMMAND_DEFSTYLE  */
+    TOK_COMMAND_DEFDESIGN = 303,   /* TOK_COMMAND_DEFDESIGN  */
+    TOK_COMMAND_DEFPROC = 304,     /* TOK_COMMAND_DEFPROC  */
+    TOK_COMMAND_REPLAY = 305,      /* TOK_COMMAND_REPLAY  */
+    TOK_COMMAND_SET = 306,         /* TOK_COMMAND_SET  */
+    TOK_COMMAND_INCLUDE = 307,     /* TOK_COMMAND_INCLUDE  */
+    TOK_COMMAND_BIG = 308,         /* TOK_COMMAND_BIG  */
+    TOK_COMMAND_BOX = 309,         /* TOK_COMMAND_BOX  */
+    TOK_COMMAND_PIPE = 310,        /* TOK_COMMAND_PIPE  */
+    TOK_COMMAND_MARK = 311,        /* TOK_COMMAND_MARK  */
+    TOK_COMMAND_MARK_SRC = 312,    /* TOK_COMMAND_MARK_SRC  */
+    TOK_COMMAND_MARK_DST = 313,    /* TOK_COMMAND_MARK_DST  */
+    TOK_COMMAND_PARALLEL = 314,    /* TOK_COMMAND_PARALLEL  */
+    TOK_COMMAND_OVERLAP = 315,     /* TOK_COMMAND_OVERLAP  */
+    TOK_COMMAND_INLINE = 316,      /* TOK_COMMAND_INLINE  */
+    TOK_VERTICAL = 317,            /* TOK_VERTICAL  */
+    TOK_VERTICAL_SHAPE = 318,      /* TOK_VERTICAL_SHAPE  */
+    TOK_AT = 319,                  /* TOK_AT  */
+    TOK_LOST = 320,                /* TOK_LOST  */
+    TOK_AT_POS = 321,              /* TOK_AT_POS  */
+    TOK_SHOW = 322,                /* TOK_SHOW  */
+    TOK_HIDE = 323,                /* TOK_HIDE  */
+    TOK_ACTIVATE = 324,            /* TOK_ACTIVATE  */
+    TOK_DEACTIVATE = 325,          /* TOK_DEACTIVATE  */
+    TOK_BYE = 326,                 /* TOK_BYE  */
+    TOK_START = 327,               /* TOK_START  */
+    TOK_BEFORE = 328,              /* TOK_BEFORE  */
+    TOK_END = 329,                 /* TOK_END  */
+    TOK_AFTER = 330,               /* TOK_AFTER  */
+    TOK_COMMAND_VSPACE = 331,      /* TOK_COMMAND_VSPACE  */
+    TOK_COMMAND_HSPACE = 332,      /* TOK_COMMAND_HSPACE  */
+    TOK_COMMAND_SYMBOL = 333,      /* TOK_COMMAND_SYMBOL  */
+    TOK_COMMAND_NOTE = 334,        /* TOK_COMMAND_NOTE  */
+    TOK_COMMAND_COMMENT = 335,     /* TOK_COMMAND_COMMENT  */
+    TOK_COMMAND_ENDNOTE = 336,     /* TOK_COMMAND_ENDNOTE  */
+    TOK_COMMAND_FOOTNOTE = 337,    /* TOK_COMMAND_FOOTNOTE  */
+    TOK_COMMAND_TITLE = 338,       /* TOK_COMMAND_TITLE  */
+    TOK_COMMAND_SUBTITLE = 339,    /* TOK_COMMAND_SUBTITLE  */
+    TOK_COMMAND_TEXT = 340,        /* TOK_COMMAND_TEXT  */
+    TOK_SHAPE_COMMAND = 341,       /* TOK_SHAPE_COMMAND  */
+    TOK_JOIN = 342,                /* TOK_JOIN  */
+    TOK_IF = 343,                  /* TOK_IF  */
+    TOK_THEN = 344,                /* TOK_THEN  */
+    TOK_ELSE = 345,                /* TOK_ELSE  */
+    TOK_MSCGEN_RBOX = 346,         /* TOK_MSCGEN_RBOX  */
+    TOK_MSCGEN_ABOX = 347,         /* TOK_MSCGEN_ABOX  */
+    TOK_UNRECOGNIZED_CHAR = 348,   /* TOK_UNRECOGNIZED_CHAR  */
+    TOK_WHITESPACE = 349,          /* TOK_WHITESPACE  */
+    TOK_NEWLINE = 350,             /* TOK_NEWLINE  */
+    TOK_COMMENT = 351,             /* TOK_COMMENT  */
+    TOK_SYMBOLPOS_ABOVE = 352,     /* TOK_SYMBOLPOS_ABOVE  */
+    TOK_SYMBOLPOS_BELOW = 353,     /* TOK_SYMBOLPOS_BELOW  */
+    TOK_SYMBOLPOS_LEFTOF = 354,    /* TOK_SYMBOLPOS_LEFTOF  */
+    TOK_SYMBOLPOS_RIGHTOF = 355,   /* TOK_SYMBOLPOS_RIGHTOF  */
+    TOK_SYMBOLPOS_AROUND = 356     /* TOK_SYMBOLPOS_AROUND  */
   };
   typedef enum msc_csh_tokentype msc_csh_token_kind_t;
 #endif
@@ -187,9 +196,9 @@ extern int msc_csh_debug;
 #if ! defined MSC_CSH_STYPE && ! defined MSC_CSH_STYPE_IS_DECLARED
 union MSC_CSH_STYPE
 {
-#line 134 "./msc_lang.yy"
+#line 136 "./msc_lang.yy"
 
-    str_view                                                  str;
+    str_view                                                   str;
     const char*                                                input_text_ptr;
     int                                                        condition; //0:false, 1:true, 2:had_error
     ECompareOperator                                           compare_op;
@@ -208,6 +217,7 @@ union MSC_CSH_STYPE
     gsl::owner<CHAR_IF_CSH(Attribute)*>                        attrib;
     gsl::owner<CHAR_IF_CSH(AttributeList)*>                    attriblist;
     gsl::owner<CHAR_IF_CSH(VertXPos)*>                         vertxpos;
+    gsl::owner<CHAR_IF_CSH(VertRefPos)*>                       vertrefpos;
     gsl::owner<CHAR_IF_CSH(ExtVertXPos)*>                      extvertxpos;
     gsl::owner<CHAR_IF_CSH(NamePair)*>                         namerel;
     gsl::owner<std::vector<std::string>*>                      stringlist;
@@ -227,7 +237,7 @@ union MSC_CSH_STYPE
     gsl::owner<CHAR_IF_CSH(ProcParamInvocationList)*>          procparaminvoclist;
     gsl::owner<CHAR_IF_CSH(ProcDefParseHelper<AttributeList>)*>procdefhelper;
 
-#line 231 "msc_parser_csh.h"
+#line 241 "msc_parser_csh.h"
 
 };
 typedef union MSC_CSH_STYPE MSC_CSH_STYPE;
