@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -23,14 +23,14 @@
 
 #define _USE_MATH_DEFINES
 #include <math.h>
-#include <map>
+#include <limits>
 #include "msccsh.h"
 #include "cgen_color.h"
-#include "utf8utils.h" //for UTF8len
 #include "stringparse.h" //for extracting csh out of strings
 #include "cgen_attribute.h"  //for CaseInsensitive compares
 #include "msc_parser_csh.h"
 #include "msc.h"
+#include "cgencommon.h"
 
 using namespace std;
 using namespace msc;
@@ -38,10 +38,22 @@ using namespace msc;
 /** Names of symbols for coloring
  *
  * All keywords shall be repeated here, not only known by the parser.
- * We only color these as keywords where a symbol name is expected (after 'symbol' keyword).*/
+ * We only color these as keywords where a symbol name is expected (after 'symbol' keyword).
+ * Note that any language name is a valid symbol name (resulting in inlining a chart), but those
+ * we have to add dynamically during runtime.*/
 static const char symbol_names[][ENUM_STRING_LEN] =
 {"arc", "rectangle", "...", "text", "cross", "shape", ""};
 
+/** Descriptions of the symbols in the order above */
+static const char* symbol_descriptions[] = {
+    "This draws a circle or ellipse.",
+    "This draws a rectangle.",
+    "This draws three small circles one below another, a kind of vertical ellipsys.",
+    "This draws just text.",
+    "This draws a larger 'X' symbol.",
+    "This draws an arbitrary shape defined via the 'defshape' command.",
+};
+
 /** Names of extended vertical position designators for coloring
  *
  * All keywords shall be repeated here, not only known by the parser.
@@ -50,6 +62,20 @@ static const char extvxpos_designator_names[][ENUM_STRING_LEN] =
 {"left", "right", "center", ""};
 
 
+template<>
+const char EnumEncapsulator<Symbol::ESymbolPosType>::names[][ENUM_STRING_LEN] =
+{ "", "above", "below", "rightof", "leftof", "around", "" };
+
+template<>
+const char* const EnumEncapsulator<Symbol::ESymbolPosType>::descriptions[] = { "",
+    "Draw above the referenced element (or the previous element if none referenced).",
+    "Draw below the referenced element (or the previous element if none referenced).",
+    "Draw right of the referenced element (or the previous element if none referenced).",
+    "Draw left of the referenced element (or the previous element if none referenced).",
+    "Draw around the referenced element (or the previous element if none referenced).",
+};
+
+
 /** Override this, since we color mscgen attributes differently. */
 void MscCsh::AddCSH_AttrName(const CshPos&pos, std::string_view name, EColorSyntaxType color)
 {
@@ -71,7 +97,7 @@ void MscCsh::AddCSH_AttrName(const CshPos&pos, std::string_view name, EColorSynt
         mscgen_compat_match = true;
     }
     //Honor partial matches only if cursor is right after
-    if (pos.last_pos != cursor_pos && match_result == 1)
+    if (pos.end_pos != cursor_pos && match_result == 1)
         match_result = 0;
     switch (match_result) {
     case 2: AddCSH(pos, mscgen_compat_match ? EColorSyntaxType(color+2) : color); return;
@@ -111,7 +137,7 @@ void MscCsh::AddCSH_StyleOrAttrName(const CshPos&pos, std::string_view name)
         mscgen_compat_match = true;
     }
 
-    if (pos.last_pos == cursor_pos && match_result_attr == 1) {
+    if (pos.end_pos == cursor_pos && match_result_attr == 1) {
         AddCSH(pos, COLOR_ATTRNAME_PARTIAL);
         was_partial = true;
         return;
@@ -125,7 +151,7 @@ void MscCsh::AddCSH_StyleOrAttrName(const CshPos&pos, std::string_view name)
         AddCSH(pos, COLOR_STYLENAME);
         return;
     }
-    if (match_result_style==1 && pos.last_pos==cursor_pos) {
+    if (match_result_style==1 && pos.end_pos==cursor_pos) {
         AddCSH(pos, COLOR_ATTRVALUE);
         was_partial = true;
         return;
@@ -137,7 +163,7 @@ void MscCsh::AddCSH_StyleOrAttrName(const CshPos&pos, std::string_view name)
 
 void MscCsh::AddCSH_EntityOrMarkerName(const CshPos&pos, std::string_view name)
 {
-    if (EntityNames.find(string(name)) != EntityNames.end())
+    if (EntityNames.find(name) != EntityNames.end())
         AddCSH(pos, COLOR_ENTITYNAME);
     else
         AddCSH(pos, COLOR_MARKERNAME);
@@ -153,7 +179,9 @@ void MscCsh::AddCSH_EntityOrMarkerName(const CshPos&pos, std::string_view name)
 void MscCsh::AddCSH_SymbolName(const CshPos&pos, std::string_view name)
 {
     unsigned match_result = FindPrefix(symbol_names, name);
-    if (pos.last_pos == cursor_pos && match_result == 1) {
+    if (match_result<2 && languages)
+        match_result = std::max(match_result, FindPrefix(languages->GetLanguages(), name));
+    if (pos.end_pos == cursor_pos && match_result == 1) {
         AddCSH(pos, COLOR_KEYWORD_PARTIAL);
         was_partial = true;
         return;
@@ -166,6 +194,9 @@ void MscCsh::AddCSH_SymbolName(const CshPos&pos, std::string_view name)
     string s = "Unknown symbol name. Use one of '";
     for (auto t: symbol_names)
         s.append(t).append("', '");
+    if (languages)
+        for (auto t : languages->GetLanguages())
+            s.append(t).append("', '");
     s.erase(s.length()-3);
     s.append(".");
     AddCSH_Error(pos, std::move(s));
@@ -189,7 +220,7 @@ void MscCsh::AddCSH_LeftRightCenterMarker(const CshPos&pos, std::string_view nam
 void MscCsh::AddCSH_ExtvxposDesignatorName(const CshPos&pos, std::string_view name)
 {
     unsigned match_result = FindPrefix(extvxpos_designator_names, name);
-    if (pos.last_pos == cursor_pos && match_result == 1) {
+    if (pos.end_pos == cursor_pos && match_result == 1) {
         AddCSH(pos, COLOR_KEYWORD_PARTIAL);
         was_partial = true;
         return;
@@ -227,9 +258,9 @@ bool msc::CshHintGraphicCallbackForMarkers(Canvas *canvas, CshHintGraphicParam /
     return true;
 }
 
-void MscCsh::BeforeYaccParse(std::string&& input, int cursor_p)
+void MscCsh::BeforeYaccParse(std::string&& input, CharByteIndex first_p, CharByteIndex cursor_p)
 {
-    Csh::BeforeYaccParse(std::move(input), cursor_p);
+    Csh::BeforeYaccParse(std::move(input), first_p, cursor_p);
     ResetMarkers();
 }
 
@@ -250,27 +281,29 @@ void MscCsh::AfterYaccParse()
 /** Parse chart text for color syntax and hint info
  *
  * @param [in] input The chart text
+ * @param [in] first_char The position of the first character of 'input' in the file we parse.
+ *             If we parse a whole file, it shall be {0,0}
  * @param [in] cursor_p The current position of the cursor.
  * @param [in] pedantic The initial value of the pedantic chart option.
    (We ignore it in CSH parsing.)*/
-void MscCsh::ParseText(std::string&& input, int cursor_p, bool pedantic)
+void MscCsh::ParseText(std::string&& input, CharByteIndex first_char, CharByteIndex cursor_p, bool pedantic [[maybe_unused]] )
 {
+    pedantic_at_start = pedantic;
     //initialize data struct
-    BeforeYaccParse(std::move(input), cursor_p);
-    (void)pedantic;
+    BeforeYaccParse(std::move(input), first_char, cursor_p);
     //call parsing
-    sv_reader<true> reader(input_text, 0);
+    sv_reader<true> reader(input_text, first_char);
     msc::lexing_state state = mscgen_compat == EMscgenCompat::FORCE_MSCGEN ? msc::lexing_state::mscgen_compat: msc::lexing_state::basic;
     msc_csh_parse(*this, state, reader); //return value ignored
     //Tidy up afterwards
     AfterYaccParse();
 }
 
-void MscCsh::ParseDesignText(std::string&& input, int cursor_p) {
+void MscCsh::ParseDesignText(std::string&& input) {
     //Design libs shall be parsed with EMscgenCompat::AUTODETECT.
     //Save & restore the value
     const EMscgenCompat saved_mscgen_compat = std::exchange(mscgen_compat, EMscgenCompat::AUTODETECT);
-    ParseText(std::move(input), cursor_p, true);
+    ParseText(std::move(input), CharByteIndex{}, CharByteIndex::invalid(), true);
     mscgen_compat = saved_mscgen_compat;
 }
 
@@ -280,8 +313,8 @@ void MscCsh::ParseDesignText(std::string&& input, int cursor_p) {
 //use defaultDesign only to get default styles and colors to produce hints.
 //Styles are polymorphic, so their AttributeNames() and AttributeValues()
 //will return even the MscStyle specific attributes, even if stored in a ContextBase.
-MscCsh::MscCsh(Csh::FileListProc proc) :
-    Csh(MscContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc),
+MscCsh::MscCsh(Csh::FileListProc proc, const LanguageCollection* languages) :
+    Csh(MscContext(true, EContextParse::NORMAL, EContextCreate::PLAIN, FileLineCol()), proc, languages),
     addMarkersAtEnd(false),
     hint_vertical_shapes(false),
     mscgen_compat(EMscgenCompat::AUTODETECT)
@@ -311,24 +344,8 @@ void MscCsh::SwitchToMscgenCompatMode()
 /** Add the symbol types to the hints.*/
 void MscCsh::AddSymbolTypesToHints()
 {
-    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "arc",
-        "This draws a circle or ellipse.",
-        EHintType::KEYWORD, true));
-    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "rectangle",
-        "This draws a rectangle.",
-        EHintType::KEYWORD, true));
-    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "...",
-        "This draws three small circles one below another, a kind of vertical ellipsys.",
-        EHintType::KEYWORD, true));
-    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "text",
-        "This draws just text.",
-        EHintType::KEYWORD, true));
-    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "cross",
-        "This draws a larger 'X' symbol.",
-        EHintType::KEYWORD, true));
-    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "shape",
-        "This draws an arbitrary shape defined via the 'defshape' command.",
-        EHintType::KEYWORD, true));
+    AddToHints(symbol_names, symbol_descriptions, HintPrefix(COLOR_KEYWORD), EHintType::KEYWORD, nullptr, true);
+    AddLanguagesToHints("symbol");
 }
 
 /** Add the left/right/center keyword (for symbol positioning) to the hints.*/
@@ -348,6 +365,40 @@ void MscCsh::AddLeftRightCenterToHints()
         CshHintGraphicParam(MSC_IDENT_CENTER)));
 }
 
+/** Callback for drawing how to draw a symbol in relation to another element in the hints popup list box.
+* @ingroup hintpopup_callbacks*/
+bool CshHintGraphicCallbackForSymbolPos(Canvas* canvas, CshHintGraphicParam p, CshHintStore&) {
+    if (!canvas) return false;
+    const Symbol::ESymbolPosType t = (Symbol::ESymbolPosType)(int)p;
+    const LineAttr elem_line(ELineType::SOLID, ColorType::black(), 1, ECornerType::NONE, 0);
+    const FillAttr elem_fill(ColorType::lgray(), EGradientType::NONE);
+    Block elem(HINT_GRAPHIC_SIZE_X / 3., HINT_GRAPHIC_SIZE_X * 2 / 3., HINT_GRAPHIC_SIZE_Y / 3., HINT_GRAPHIC_SIZE_Y * 2 / 3.);
+    canvas->Fill(elem, elem_fill);
+    canvas->Line(elem, elem_line);
+
+    const LineAttr line(ELineType::SOLID, ColorType::red(), 1, ECornerType::NONE, 0);
+    const double radius_div = t == Symbol::ESymbolPosType::AROUND ? 3. : 8.;
+    static_assert(std::numeric_limits<double>::has_quiet_NaN);
+    const XY center = ([t]() { switch (t) {
+        case Symbol::ESymbolPosType::ABOVE: return XY( 0, -1);
+        case Symbol::ESymbolPosType::BELOW: return XY( 0, +1);
+        case Symbol::ESymbolPosType::RIGHTOF: return XY(+1, 0);
+        case Symbol::ESymbolPosType::LEFTOF: return XY(-1, 0);
+        case Symbol::ESymbolPosType::AROUND: return XY(0, 0);
+        default: return XY(std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN());
+    }}()*0.4 + XY(0.5, 0.5)).Scale(XY(HINT_GRAPHIC_SIZE_X, HINT_GRAPHIC_SIZE_Y));
+    if (std::isnan(center.x)) return false;
+    const Contour circle(center, HINT_GRAPHIC_SIZE_X / radius_div, HINT_GRAPHIC_SIZE_Y / radius_div);
+    canvas->Line(circle, line);
+    return true;
+}
+
+/** Add the leftof/rightof/above/below/around keyword (for symbol positioning) to the hints.*/
+void MscCsh::AddSymbolPosToHints() {
+    AddToHints(EnumEncapsulator<Symbol::ESymbolPosType>::names, EnumEncapsulator<Symbol::ESymbolPosType>::descriptions, 
+               HintPrefix(COLOR_KEYWORD), EHintType::KEYWORD, CshHintGraphicCallbackForSymbolPos, true);
+}
+
 /** Add the left/right keyword (for hspace specification) to the hints.*/
 void MscCsh::AddLeftRightHSpaceToHints()
 {
@@ -359,7 +410,6 @@ void MscCsh::AddLeftRightHSpaceToHints()
         EHintType::KEYWORD));
 }
 
-
 /** Add the vertical types to the hints.*/
 void MscCsh::AddVerticalTypesToHints()
 {
@@ -367,7 +417,7 @@ void MscCsh::AddVerticalTypesToHints()
         "Use this to add a large vertical curly brace, like '}'.",
         EHintType::KEYWORD));
     AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "bracket",
-        "Use this to add a large square baracket, like ']'.",
+        "Use this to add a large square bracket, like ']'.",
         EHintType::KEYWORD));
     AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "range",
         "Use this to mark a vertical range, like this 'I'.",
@@ -383,6 +433,17 @@ void MscCsh::AddVerticalTypesToHints()
         EHintType::KEYWORD));
 }
 
+/** Add the left/right keyword (for hspace specification) to the hints.*/
+void MscCsh::AddLeftRightOfToHints() {
+    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "leftof",
+                       "Draw left of the referenced element (or the previous element if none referenced).",
+                       EHintType::KEYWORD));
+    AddToHints(CshHint(HintPrefix(COLOR_KEYWORD) + "rightof",
+                       "Draw right of the referenced element (or the previous element if none referenced).",
+                       EHintType::KEYWORD));
+}
+
+
 /** Add entities defined up to now to the list of hints. */
 void MscCsh::AddEntitiesToHints()
 {
@@ -436,8 +497,8 @@ void MscCsh::AddKeywordsToHints(bool includeParallel)
         "overlap", "If you prepend 'overlap' in front of an element, subsequent elements will be laid besides it even if overlap occurs. Useful for slanted arrows crossing each other.",
         "box", "Use before a box definition as syntactic sugar.",
         "block", "Use before an an arrow definition to create a block arrow, like 'block a->b;'.",
-        "pipe", "Create a pipe between two entitie.",
-        "nudge", "Add a bit of a vertical space to pust subsequent elements lower.",
+        "pipe", "Create a pipe between two entities.",
+        "nudge", "Add a bit of a vertical space to push subsequent elements lower.",
         "heading", "Force the display of an entity heading for each currently showing entity.",
         "newpage", "Insert a page break here.",
         "set", "Define or set the value of a variable or change the value of a parameter inside a procedure.",
@@ -449,7 +510,7 @@ void MscCsh::AddKeywordsToHints(bool includeParallel)
         "show", "Turn on entities, so that they become visible.",
         "hide", "Turn off entities, so that they do not show from now on.",
         "activate", "Activate entities, so that their entity line becomes wider.",
-        "deactivate", "Deactivate entities, swicthing back to a single entity line.",
+        "deactivate", "Deactivate entities, switching back to a single entity line.",
         "bye", "Anything specified after this command is treated as comment and is ignored.",
         "hspace", "Add (extra) horizontal spacing between two entities.",
         "vspace", "Add vertical space pushing subsequent elements lower.",
@@ -461,7 +522,7 @@ void MscCsh::AddKeywordsToHints(bool includeParallel)
         "title", "Add a title to the chart.",
         "subtitle", "Add a subtitle to a section of the chart.",
         "defshape", "Define a new shape.",
-        "text at", "Draws free text.",
+        "text", "Draws free text.",
         "defproc", "Define a new procedure that can be re-played later.",
         "include", "Include the text of another file in parsing.",
         "if", "Start a conditional statement. Useful only inside procedures.",
@@ -617,7 +678,7 @@ void MscCsh::FillNamesHints()
     Newpage::AttributeNames(*this);
     Note::AttributeNames(*this, true);
     Note::AttributeNames(*this, false);
-    Symbol::AttributeNames(*this);
+    Symbol::AttributeNames(*this, 2);
     AddVSpace::AttributeNames(*this);
     EntityApp::AttributeNames(*this);
     //At this point we do not have shapes read from the default lib, so
