@@ -318,7 +318,7 @@ int main(int argc, char* argv[])
 #endif //BUILD_GUI
 
     //Do it
-    auto ret = do_main(args, std::move(languages), std::move(design_files),
+    auto ret = do_main(args, languages, std::move(design_files),
                        OpenNamedFile, ReadIncludeFile, nullptr,
                        "\\mn(10)", display_only, progress, NULL, &load_data);
     if (std::holds_alternative<int>(ret)) {
