@@ -874,7 +874,194 @@ void contour_unit_tests(bool do_image) {
         //We have made ASSERT tolerance adaptive in this case to avoid triggering the assert.
     }
 
-    //Now print results
+    //This tests an Untangle case, when the rightmost point of the path is a CP and has only incoming edges on the outside.
+    {
+        //   1--------2            In this untangle operation
+        //   |         \           the input path goes by numbered vertices.
+        //   |          \          The edge 3->4 overlaps with 6->7 and likewise
+        //   |     7,4===3,6       4->5 overlaps with 7->8. In this case the rightmost
+        //   |      ||  /          point of the path is in vertex 3 (=6). The first ray
+        //   |      || /           of this CP is the 5->6 (closest to, but after the X axis).
+        //   |      ||/            Here both 2->3 and 5->6 are incoming.
+        //   9-----8,5
+        Path p = {XY(10,0), XY(20,10), XY(15,10), XY(15, 15), XY(20,10), XY(15,10), XY(15, 15), XY(5, 25), XY(10,0)};
+        Contour c(p);
+        Contour ref(XY(20, 10), XY(5, 25), XY(10, 0));
+        BAssert(R, c == ref, "Untangle with rightmost CP having 2 incoming edges - 1");
+    }
+
+    {
+        //Same situation as the above
+        Path p = {
+            Edge(XY(0x1.9000000000000p+5,0x1.a800000000000p+4), XY(0x1.97cf7bffeb4e3p+5,0x1.59e52800cef25p+4)),
+            Edge(XY(0x1.97cf7bffeb4e3p+5,0x1.59e52800cef25p+4), XY(0x1.9958a9f992361p+5,0x1.4e7d16aefd7f2p+4)),
+            Edge(XY(0x1.9958a9f992361p+5,0x1.4e7d16aefd7f2p+4), XY(0x1.9b0ea10f7da8ep+5,0x1.43630f0dc8b12p+4)),
+            Edge(XY(0x1.9b0ea10f7da8ep+5,0x1.43630f0dc8b12p+4), XY(0x1.97cf7bffeb4e3p+5,0x1.59e52800cef25p+4)),
+            Edge(XY(0x1.97cf7bffeb4e3p+5,0x1.59e52800cef25p+4), XY(0x1.9958a9f992361p+5,0x1.4e7d16aefd7f2p+4)),
+            Edge(XY(0x1.9958a9f992361p+5,0x1.4e7d16aefd7f2p+4), XY(0x1.9b0ea10f7da8ep+5,0x1.43630f0dc8b12p+4)),
+            Edge(XY(0x1.9b0ea10f7da8ep+5,0x1.43630f0dc8b12p+4), XY(0x1.c0a2bb64e2390p+5,0x1.966cf3fa8efdep+4), false),
+            Edge(XY(0x1.c0a2bb64e2390p+5,0x1.966cf3fa8efdep+4), XY(0x1.bbf953a9ce21bp+5,0x1.972b61b4cfddcp+4)),
+            Edge(XY(0x1.bbf953a9ce21bp+5,0x1.972b61b4cfddcp+4), XY(0x1.b70d6bff9886dp+5,0x1.986108002963bp+4)),
+            Edge(XY(0x1.b70d6bff9886dp+5,0x1.986108002963bp+4), XY(0x1.c0a2bb64e2390p+5,0x1.966cf3fa8efdep+4)),
+            Edge(XY(0x1.c0a2bb64e2390p+5,0x1.966cf3fa8efdep+4), XY(0x1.bbf953a9ce21bp+5,0x1.972b61b4cfddcp+4)),
+            Edge(XY(0x1.bbf953a9ce21bp+5,0x1.972b61b4cfddcp+4), XY(0x1.b70d6bff9886dp+5,0x1.986108002963bp+4)),
+            Edge(XY(0x1.b70d6bff9886dp+5,0x1.986108002963bp+4), XY(0x1.9000000000000p+5,0x1.a800000000000p+4)),
+        };
+        Contour c(p);
+        BAssert(R, !c.IsEmpty(), "Untangle with rightmost CP having 2 incoming edges - 2");
+    }
+
+    {
+        //Edge #9 of A and Edge #7 of B almost overlap, but are further than 1e-5, which is the cutoff
+        //to merge crosspoints. Their one end (endpoint of A's #9 and startpoint of B's #7) are detected as a
+        //crosspoint (because of A's #10 and B's #6), but the other end is not. This sends walking haywire.
+        Contour A = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.14f0d5866936fp+7,0x1.fafaeb6b74637p+5), XY(0x1.1212fcf57c93cp+7,0x1.f241a5bc7c809p+5)),
+            Edge(XY(0x1.1212fcf57c93cp+7,0x1.f241a5bc7c809p+5), XY(0x1.0ec7782864c5dp+7,0x1.e9184e502cdbbp+5)),
+            Edge(XY(0x1.0ec7782864c5dp+7,0x1.e9184e502cdbbp+5), XY(0x1.0b62fec0aa666p+7,0x1.e083cc742677ap+5)),
+            Edge(XY(0x1.0b62fec0aa666p+7,0x1.e083cc742677ap+5), XY(0x1.0ad998ded2ae3p+7,0x1.d4fb9d19ee699p+5), false, true),
+            Edge(XY(0x1.0ad998ded2ae3p+7,0x1.d4fb9d19ee699p+5), XY(0x1.0d37f2a76944fp+7,0x1.c83a43971fef4p+5)),
+            Edge(XY(0x1.0d37f2a76944fp+7,0x1.c83a43971fef4p+5), XY(0x1.0f76c6a40778fp+7,0x1.baf9af76c5a21p+5)),
+            Edge(XY(0x1.0f76c6a40778fp+7,0x1.baf9af76c5a21p+5), XY(0x1.119c0cc5fbc8fp+7,0x1.ad16c4ea9432fp+5)),
+            Edge(XY(0x1.119c0cc5fbc8fp+7,0x1.ad16c4ea9432fp+5), XY(0x1.0fec884ed12a5p+7,0x1.98cf865ef6b5fp+5)),
+            Edge(XY(0x1.0fec884ed12a5p+7,0x1.98cf865ef6b5fp+5), XY(0x1.1393d84965bffp+7,0x1.9a761b6657dadp+5), true, true),
+            Edge(XY(0x1.1393d84965bffp+7,0x1.9a761b6657dadp+5), XY(0x1.1753798d18891p+7,0x1.a00b6faf64428p+5), true, true),
+            Edge(XY(0x1.1753798d18891p+7,0x1.a00b6faf64428p+5), XY(0x1.1bc5ed36361edp+7,0x1.fffe9b9b83896p+5), XY(0x1.19b666e1ba0f5p+7,0x1.bf9e2b72e538fp+5), XY(0x1.1b35355a62be7p+7,0x1.e0005c49f9e12p+5)),
+            Edge(XY(0x1.1bc5ed36361edp+7,0x1.fffe9b9b83896p+5), XY(0x1.18207d517c4abp+7,0x1.f96a76de524f9p+5), true, true),
+            Edge(XY(0x1.18207d517c4abp+7,0x1.f96a76de524f9p+5), XY(0x1.1bf1dc3427ba0p+7,0x1.061c51068c50cp+6)),
+            Edge(XY(0x1.1bf1dc3427ba0p+7,0x1.061c51068c50cp+6), XY(0x1.1c09a244c9dccp+7,0x1.0ef06ede65250p+6)),
+            Edge(XY(0x1.1c09a244c9dccp+7,0x1.0ef06ede65250p+6), XY(0x1.1aaf4e7ab2d08p+7,0x1.30c3477bf0aebp+6), XY(0x1.1c09b0a40dd3ap+7,0x1.1a38ee2ae9906p+6), XY(0x1.1b9512fb953f6p+7,0x1.259b945299d25p+6)),
+            Edge(XY(0x1.1aaf4e7ab2d08p+7,0x1.30c3477bf0aebp+6), XY(0x1.1729ed0701aebp+7,0x1.2cf77fe9cdcd7p+6), true, true),
+            Edge(XY(0x1.1729ed0701aebp+7,0x1.2cf77fe9cdcd7p+6), XY(0x1.139bcb61208e4p+7,0x1.2eb04e5f50aabp+6), true, true),
+            Edge(XY(0x1.139bcb61208e4p+7,0x1.2eb04e5f50aabp+6), XY(0x1.14987290aa51cp+7,0x1.2bdfb63ab2e24p+6)),
+            Edge(XY(0x1.14987290aa51cp+7,0x1.2bdfb63ab2e24p+6), XY(0x1.12011ed6f762ep+7,0x1.259b14adcff32p+6)),
+            Edge(XY(0x1.12011ed6f762ep+7,0x1.259b14adcff32p+6), XY(0x1.0f52c963a987cp+7,0x1.1faba5fcd9a03p+6)),
+            Edge(XY(0x1.0f52c963a987cp+7,0x1.1faba5fcd9a03p+6), XY(0x1.0c860f32ff914p+7,0x1.1a014f8737db6p+6)),
+            Edge(XY(0x1.0c860f32ff914p+7,0x1.1a014f8737db6p+6), XY(0x1.0ca9c02e4c468p+7,0x1.141ff8cf7354dp+6), false, true),
+            Edge(XY(0x1.0ca9c02e4c468p+7,0x1.141ff8cf7354dp+6), XY(0x1.0f0af6e696496p+7,0x1.1025017b6766dp+6)),
+            Edge(XY(0x1.0f0af6e696496p+7,0x1.1025017b6766dp+6), XY(0x1.1151825a23908p+7,0x1.0c3618c17b9f8p+6)),
+            Edge(XY(0x1.1151825a23908p+7,0x1.0c3618c17b9f8p+6), XY(0x1.1580ecc6811dap+7,0x1.03be74de4f6d1p+6)),
+            Edge(XY(0x1.1580ecc6811dap+7,0x1.03be74de4f6d1p+6), XY(0x1.15244643262bdp+7,0x1.ff7207c17ad51p+5)),
+            Edge(XY(0x1.15244643262bdp+7,0x1.ff7207c17ad51p+5), XY(0x1.14de6ea226627p+7,0x1.ffff1921fdfabp+5), true, true),
+            Edge(XY(0x1.14de6ea226627p+7,0x1.ffff1921fdfabp+5), XY(0x1.1514c54340deep+7,0x1.fe198072ebec0p+5)),
+            Edge(XY(0x1.1514c54340deep+7,0x1.fe198072ebec0p+5), XY(0x1.14f0d5866936fp+7,0x1.fafaeb6b74637p+5)),
+        }), ContourList::UnsafeMake({})), ContourList::UnsafeMake({}));
+        Contour B = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.047409b0b24dbp+7,0x1.848fb62b3d7a7p+5), XY(0x1.061452c42fc53p+7,0x1.7609d0194e4dfp+5)),
+            Edge(XY(0x1.061452c42fc53p+7,0x1.7609d0194e4dfp+5), XY(0x1.079415fde7ae5p+7,0x1.671e950de53b6p+5)),
+            Edge(XY(0x1.079415fde7ae5p+7,0x1.671e950de53b6p+5), XY(0x1.08f6e2a6419a9p+7,0x1.57aa286810e9ap+5)),
+            Edge(XY(0x1.08f6e2a6419a9p+7,0x1.57aa286810e9ap+5), XY(0x1.066b7bd7dee23p+7,0x1.42d96ccb42efdp+5)),
+            Edge(XY(0x1.066b7bd7dee23p+7,0x1.42d96ccb42efdp+5), XY(0x1.0a14450c9fedap+7,0x1.4189dbc70f2a0p+5), true, true),
+            Edge(XY(0x1.0a14450c9fedap+7,0x1.4189dbc70f2a0p+5), XY(0x1.0de883682b048p+7,0x1.462a6a7927613p+5), true, true),
+            Edge(XY(0x1.0de883682b048p+7,0x1.462a6a7927613p+5), XY(0x1.175380dd20abbp+7,0x1.a00bd06f09fd1p+5), XY(0x1.11df717fcc256p+7,0x1.63773d21f6f76p+5), XY(0x1.1509ad121b6a0p+7,0x1.81c5100fcc04dp+5)),
+            Edge(XY(0x1.175380dd20abbp+7,0x1.a00bd06f09fd1p+5), XY(0x1.1393dfb18b44bp+7,0x1.9a767b22f23bcp+5), true, true),
+            Edge(XY(0x1.1393dfb18b44bp+7,0x1.9a767b22f23bcp+5), XY(0x1.10c4e236a1390p+7,0x1.a3bddab262dcbp+5), true, true),
+            Edge(XY(0x1.10c4e236a1390p+7,0x1.a3bddab262dcbp+5), XY(0x1.10d1313c12defp+7,0x1.a1f5edeb51a22p+5)),
+            Edge(XY(0x1.10d1313c12defp+7,0x1.a1f5edeb51a22p+5), XY(0x1.0d1f919f8532ep+7,0x1.9b249a71dabc0p+5)),
+            Edge(XY(0x1.0d1f919f8532ep+7,0x1.9b249a71dabc0p+5), XY(0x1.096427a9f5344p+7,0x1.94f3f829faf3dp+5)),
+            Edge(XY(0x1.096427a9f5344p+7,0x1.94f3f829faf3dp+5), XY(0x1.059610985165bp+7,0x1.8f5630fc4cfaap+5)),
+            Edge(XY(0x1.059610985165bp+7,0x1.8f5630fc4cfaap+5), XY(0x1.047409b0b24dbp+7,0x1.848fb62b3d7a7p+5), false, true),
+        }), ContourList::UnsafeMake({})), ContourList::UnsafeMake({}));
+
+        A.Scale(30);
+        B.Scale(30);
+
+        UnionAssert(R, A, B, "Union of two contours with missing crosspoint");
+    }
+
+    {
+        Contour Res = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.830dcb553d5d6p+9,0x1.94ec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.aaec1d5734babp+7)),
+            Edge(XY(0x1.830dcb553d5d6p+9,0x1.aaec1d5734babp+7), XY(0x1.804dcb553d5d6p+9,0x1.aaec1d5734babp+7)),
+            Edge(XY(0x1.804dcb553d5d6p+9,0x1.aaec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.9fec1d5734babp+7), XY(0x1.81d29a4cba28ap+9,0x1.aaec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.a5ff593527e7cp+7)),
+            Edge(XY(0x1.830dcb553d5d6p+9,0x1.9fec1d5734babp+7), XY(0x1.804dcb553d5d6p+9,0x1.94ec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.99d8e179418dap+7), XY(0x1.81d29a4cba28ap+9,0x1.94ec1d5734babp+7)),
+            Edge(XY(0x1.804dcb553d5d6p+9,0x1.94ec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.94ec1d5734babp+7)),
+        }), ContourList::UnsafeMake({})), ContourList::UnsafeMake({}));
+        Contour A = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.85cdcb553d5d6p+9,0x1.94ec1d5734babp+7), XY(0x1.85cdcb553d5d6p+9,0x1.aaec1d5734babp+7)),
+            Edge(XY(0x1.85cdcb553d5d6p+9,0x1.aaec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.aaec1d5734babp+7)),
+            Edge(XY(0x1.830dcb553d5d6p+9,0x1.aaec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.94ec1d5734babp+7)),
+            Edge(XY(0x1.830dcb553d5d6p+9,0x1.94ec1d5734babp+7), XY(0x1.85cdcb553d5d6p+9,0x1.94ec1d5734babp+7)),
+        }), ContourList::UnsafeMake({})), ContourList::UnsafeMake({}));
+        Contour B = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.804dcb553d5d6p+9,0x1.94ec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.94ec1d5734babp+7)),
+            Edge(XY(0x1.830dcb553d5d6p+9,0x1.94ec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.aaec1d5734babp+7)),
+            Edge(XY(0x1.830dcb553d5d6p+9,0x1.aaec1d5734babp+7), XY(0x1.804dcb553d5d6p+9,0x1.aaec1d5734babp+7)),
+            Edge(XY(0x1.804dcb553d5d6p+9,0x1.aaec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.9fec1d5734babp+7), XY(0x1.81d29a4cba28ap+9,0x1.aaec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.a5ff593527e7cp+7)),
+            Edge(XY(0x1.830dcb553d5d6p+9,0x1.9fec1d5734babp+7), XY(0x1.804dcb553d5d6p+9,0x1.94ec1d5734babp+7), XY(0x1.830dcb553d5d6p+9,0x1.99d8e179418dap+7), XY(0x1.81d29a4cba28ap+9,0x1.94ec1d5734babp+7)),
+        }), ContourList::UnsafeMake({})), ContourList::UnsafeMake({}));
+        
+        UnionAssert(R, A, B, "Union of two contours 2");
+    }
+
+    {
+        Contour Res = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.837234aac2a34p+9,0x1.8d13e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.a313e2a8cb45dp+7)),
+            Edge(XY(0x1.837234aac2a34p+9,0x1.a313e2a8cb45dp+7), XY(0x1.80b234aac2a34p+9,0x1.a313e2a8cb45dp+7)),
+            Edge(XY(0x1.80b234aac2a34p+9,0x1.a313e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.9813e2a8cb45dp+7), XY(0x1.823703a23f6e8p+9,0x1.a313e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.9e271e86be72ep+7)),
+            Edge(XY(0x1.837234aac2a34p+9,0x1.9813e2a8cb45dp+7), XY(0x1.80b234aac2a34p+9,0x1.8d13e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.9200a6cad818cp+7), XY(0x1.823703a23f6e8p+9,0x1.8d13e2a8cb45dp+7)),
+            Edge(XY(0x1.80b234aac2a34p+9,0x1.8d13e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.8d13e2a8cb45dp+7)),
+        }), ContourList::UnsafeMake({})), ContourList::UnsafeMake({}));
+        Contour A = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.863234aac2a34p+9,0x1.8d13e2a8cb45dp+7), XY(0x1.863234aac2a34p+9,0x1.a313e2a8cb45dp+7)),
+            Edge(XY(0x1.863234aac2a34p+9,0x1.a313e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.a313e2a8cb45dp+7)),
+            Edge(XY(0x1.837234aac2a34p+9,0x1.a313e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.8d13e2a8cb45dp+7)),
+            Edge(XY(0x1.837234aac2a34p+9,0x1.8d13e2a8cb45dp+7), XY(0x1.863234aac2a34p+9,0x1.8d13e2a8cb45dp+7)),
+        }), ContourList::UnsafeMake({})), ContourList::UnsafeMake({}));
+        Contour B = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.80b234aac2a34p+9,0x1.8d13e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.8d13e2a8cb45dp+7)),
+            Edge(XY(0x1.837234aac2a34p+9,0x1.8d13e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.a313e2a8cb45dp+7)),
+            Edge(XY(0x1.837234aac2a34p+9,0x1.a313e2a8cb45dp+7), XY(0x1.80b234aac2a34p+9,0x1.a313e2a8cb45dp+7)),
+            Edge(XY(0x1.80b234aac2a34p+9,0x1.a313e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.9813e2a8cb45dp+7), XY(0x1.823703a23f6e8p+9,0x1.a313e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.9e271e86be72ep+7)),
+            Edge(XY(0x1.837234aac2a34p+9,0x1.9813e2a8cb45dp+7), XY(0x1.80b234aac2a34p+9,0x1.8d13e2a8cb45dp+7), XY(0x1.837234aac2a34p+9,0x1.9200a6cad818cp+7), XY(0x1.823703a23f6e8p+9,0x1.8d13e2a8cb45dp+7)),
+        }), ContourList::UnsafeMake({})), ContourList::UnsafeMake({}));
+        UnionAssert(R, A, B, "Union of two contours 3");
+    }
+
+    {
+        //Here we have a tricky crosspoint
+        //                |(a)
+        //                |     The crosspoint at the star is found well. However the edeges marked as (a) and (b) are not connected
+        //       ....<....*.+   are very close to each other: in the X dimension they are (a)=37.171875000000000, while (b)=37.171875000000007.
+        //  --<-----------+ :   (The other 2edges shown differ in Y b 0.1 pixels.) This small difference means that (a) and (b) are not
+        //                  :   crossing as their hull does not cross, but the * crosspoint will be filed to the vertex of the dotted contour.
+        //               (b):   This prompts us to make HullOverlap() a bit less conservative.
+
+
+        Contour A = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.b2c0000000000p+4,0x1.5acccccccccccp+6), XY(0x1.57c6666666667p+5,0x1.5acccccccccccp+6)),
+            Edge(XY(0x1.57c6666666667p+5,0x1.5acccccccccccp+6), XY(0x1.57c6666666667p+5,0x1.ce66666666666p+6)),
+            Edge(XY(0x1.57c6666666667p+5,0x1.ce66666666666p+6), XY(0x1.8ce2666666666p+6,0x1.ce66666666666p+6)),
+            Edge(XY(0x1.8ce2666666666p+6,0x1.ce66666666666p+6), XY(0x1.8ce2666666666p+6,0x1.b000000000000p+6)),
+            Edge(XY(0x1.8ce2666666666p+6,0x1.b000000000000p+6), XY(0x1.a41599999999ap+6,0x1.b000000000000p+6), true, true),
+            Edge(XY(0x1.a41599999999ap+6,0x1.b000000000000p+6), XY(0x1.a41599999999ap+6,0x1.e59999999999ap+6)),
+            Edge(XY(0x1.a41599999999ap+6,0x1.e59999999999ap+6), XY(0x1.2960000000001p+5,0x1.e59999999999ap+6)),
+            Edge(XY(0x1.2960000000001p+5,0x1.e59999999999ap+6), XY(0x1.2960000000001p+5,0x1.8b9999999999ap+6)),
+            Edge(XY(0x1.2960000000001p+5,0x1.8b9999999999ap+6), XY(0x1.b2c0000000000p+4,0x1.8b9999999999ap+6)),
+            Edge(XY(0x1.b2c0000000000p+4,0x1.8b9999999999ap+6), XY(0x1.b2c0000000000p+4,0x1.b200000000000p+6)),
+            Edge(XY(0x1.b2c0000000000p+4,0x1.b200000000000p+6), XY(0x1.3000000000000p+3,0x1.b200000000000p+6)),
+            Edge(XY(0x1.3000000000000p+3,0x1.b200000000000p+6), XY(0x1.3000000000000p+3,0x1.4e00000000000p+6)),
+            Edge(XY(0x1.3000000000000p+3,0x1.4e00000000000p+6), XY(0x1.b2c0000000000p+4,0x1.4e00000000000p+6)),
+            Edge(XY(0x1.b2c0000000000p+4,0x1.4e00000000000p+6), XY(0x1.b2c0000000000p+4,0x1.5acccccccccccp+6)),
+        }),ContourList::UnsafeMake({HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(0x1.b2c0000000000p+4,0x1.7200000000000p+6), XY(0x1.b2c0000000000p+4,0x1.7466666666666p+6)),
+            Edge(XY(0x1.b2c0000000000p+4,0x1.7466666666666p+6), XY(0x1.2960000000001p+5,0x1.7466666666666p+6)),
+            Edge(XY(0x1.2960000000001p+5,0x1.7466666666666p+6), XY(0x1.2960000000001p+5,0x1.7200000000000p+6)),
+            Edge(XY(0x1.2960000000001p+5,0x1.7200000000000p+6), XY(0x1.b2c0000000000p+4,0x1.7200000000000p+6)),
+        }),ContourList::UnsafeMake({})), })), ContourList::UnsafeMake({}));
+        Contour B = Contour::UnsafeMake(HoledSimpleContour::UnsafeMake(SimpleContour::UnsafeMake(Path{
+            Edge(XY(-0x1.0000000000000p-1,0x1.2600000000000p+6), XY(0x1.2960000000000p+5,0x1.2600000000000p+6)),
+            Edge(XY(0x1.2960000000000p+5,0x1.2600000000000p+6), XY(0x1.2960000000000p+5,0x1.da00000000000p+6)),
+            Edge(XY(0x1.2960000000000p+5,0x1.da00000000000p+6), XY(-0x1.0000000000000p-1,0x1.da00000000000p+6)),
+            Edge(XY(-0x1.0000000000000p-1,0x1.da00000000000p+6), XY(-0x1.0000000000000p-1,0x1.2600000000000p+6)),
+        }),ContourList::UnsafeMake({})),ContourList::UnsafeMake({}));
+        const Block B1 = A.GetBoundingBox() + B.GetBoundingBox();
+        Contour C = A + B;
+        const Block B2 = C.GetBoundingBox();
+        BAssert(R, B1==B2, "Union with tricky crosspoint at 37.1719, 118.5");
+    }
+
+    //Now print results (to stderr)
     const size_t failed = std::ranges::count(R, false, &std::pair<std::string, bool>::second);
     if (do_image) {
         R.emplace(R.begin(), failed ? "The following unit tests failed:" : "All unit tests passed!", failed==0);
@@ -891,8 +1078,8 @@ void contour_unit_tests(bool do_image) {
                 y += 6;
             }
     } else {
-        std::cout<<(failed ? "The following unit tests failed:" : "All unit tests passed!")<<std::endl;
+        if (failed) std::cerr << "The following " << failed << " unit tests failed : " << std::endl;
         for (auto& [msg, success] : R)
-            if (!success) std::cout<<msg<<std::endl;
+            if (!success) std::cerr<<msg<<std::endl;
     }
 }
